@@ -37,6 +37,8 @@
   case kPUBLIC: \
   case kPRIVATE: \
   case kUNTYPED: \
+  case kUSE: \
+  case kAS: \
   /* nop */
 
 typedef struct {
@@ -537,8 +539,8 @@ EOP:
 }
 
 /*
-  optinal ::= {} <simple_type>
-            | {} simple_type <`?`>
+  optional ::= {} <simple_type>
+             | {} simple_type <`?`>
 */
 static VALUE parse_optional(parserstate *state) {
   range rg;
@@ -566,13 +568,30 @@ static void initialize_method_params(method_params *params){
 }
 
 /*
-  function ::= {} `(` params `)` `{` `(` params `)` `->` optional `}` `->` <optional>
-             | {} `(` params `)` `->` <optional>
-             | {} `{` `(` params `)` `->` optional `}` `->` <optional>
-             | {} `{` `->` optional `}` `->` <optional>
-             | {} `->` <optional>
+  self_type_binding ::= {} <>
+                      | {} `[` `self` `:` type <`]`>
 */
-static void parse_function(parserstate *state, VALUE *function, VALUE *block) {
+static VALUE parse_self_type_binding(parserstate *state) {
+  if (state->next_token.type == pLBRACKET) {
+    parser_advance(state);
+    parser_advance_assert(state, kSELF);
+    parser_advance_assert(state, pCOLON);
+    VALUE type = parse_type(state);
+    parser_advance_assert(state, pRBRACKET);
+    return type;
+  } else {
+    return Qnil;
+  }
+}
+
+/*
+  function ::= {} `(` params `)` self_type_binding? `{` `(` params `)` self_type_binding? `->` optional `}` `->` <optional>
+             | {} `(` params `)` self_type_binding? `->` <optional>
+             | {} self_type_binding? `{` `(` params `)` self_type_binding? `->` optional `}` `->` <optional>
+             | {} self_type_binding? `{` self_type_binding `->` optional `}` `->` <optional>
+             | {} self_type_binding? `->` <optional>
+*/
+static void parse_function(parserstate *state, VALUE *function, VALUE *block, VALUE *function_self_type) {
   method_params params;
   initialize_method_params(&params);
 
@@ -582,6 +601,11 @@ static void parse_function(parserstate *state, VALUE *function, VALUE *block) {
     parser_advance_assert(state, pRPAREN);
   }
 
+  // Passing NULL to function_self_type means the function itself doesn't accept self type binding. (== method type)
+  if (function_self_type) {
+    *function_self_type = parse_self_type_binding(state);
+  }
+
   VALUE required = Qtrue;
   if (state->next_token.type == pQUESTION && state->next_token2.type == pLBRACE) {
     // Optional block
@@ -600,6 +624,8 @@ static void parse_function(parserstate *state, VALUE *function, VALUE *block) {
       parser_advance_assert(state, pRPAREN);
     }
 
+    VALUE block_self_type = parse_self_type_binding(state);
+
     parser_advance_assert(state, pARROW);
     VALUE block_return_type = parse_optional(state);
 
@@ -614,7 +640,8 @@ static void parse_function(parserstate *state, VALUE *function, VALUE *block) {
         block_params.rest_keywords,
         block_return_type
       ),
-      required
+      required,
+      block_self_type
     );
 
     parser_advance_assert(state, pRBRACE);
@@ -642,11 +669,12 @@ static VALUE parse_proc_type(parserstate *state) {
   position start = state->current_token.range.start;
   VALUE function = Qnil;
   VALUE block = Qnil;
-  parse_function(state, &function, &block);
+  VALUE proc_self = Qnil;
+  parse_function(state, &function, &block, &proc_self);
   position end = state->current_token.range.end;
   VALUE loc = rbs_location_pp(state->buffer, &start, &end);
 
-  return rbs_proc(function, block, loc);
+  return rbs_proc(function, block, loc, proc_self);
 }
 
 /**
@@ -662,6 +690,10 @@ static VALUE parse_proc_type(parserstate *state) {
 VALUE parse_record_attributes(parserstate *state) {
   VALUE hash = rb_hash_new();
 
+  if (state->next_token.type == pRBRACE) {
+    return hash;
+  }
+
   while (true) {
     VALUE key;
     VALUE type;
@@ -919,6 +951,9 @@ static VALUE parse_simple(parserstate *state) {
 
     return rbs_tuple(types, rbs_new_location(state->buffer, rg));
   }
+  case pAREF_OPR: {
+    return rbs_tuple(rb_ary_new(), rbs_new_location(state->buffer, state->current_token.range));
+  }
   case pLBRACE: {
     position start = state->current_token.range.start;
     VALUE fields = parse_record_attributes(state);
@@ -1001,7 +1036,6 @@ VALUE parse_type(parserstate *state) {
 
   type_param ::= tUIDENT                            (module_type_params == false)
 */
-
 VALUE parse_type_params(parserstate *state, range *rg, bool module_type_params) {
   VALUE params = rb_ary_new();
 
@@ -1123,7 +1157,7 @@ VALUE parse_method_type(parserstate *state) {
 
   type_range.start = state->next_token.range.start;
 
-  parse_function(state, &function, &block);
+  parse_function(state, &function, &block, NULL);
 
   rg.end = state->current_token.range.end;
   type_range.end = rg.end;
@@ -1244,7 +1278,7 @@ VALUE parse_type_decl(parserstate *state, position comment_pos, VALUE annotation
 
   parser_pop_typevar_table(state);
 
-  return rbs_ast_decl_alias(
+  return rbs_ast_decl_type_alias(
     typename,
     type_params,
     type,
@@ -1376,6 +1410,7 @@ VALUE parse_method_name(parserstate *state, range *range) {
   case pSTAR:
   case pSTAR2:
   case pLT:
+  case pAREF_OPR:
   case tOPERATOR:
     *range = state->current_token.range;
     return ID2SYM(INTERN_TOKEN(state, state->current_token));
@@ -1451,7 +1486,7 @@ VALUE parse_member_def(parserstate *state, bool instance_only, bool accept_overl
   range keyword_range;
   range name_range;
   range kind_range;
-  range overload_range = NULL_RANGE;
+  range overloading_range = NULL_RANGE;
 
   VALUE visibility;
 
@@ -1490,8 +1525,8 @@ VALUE parse_member_def(parserstate *state, bool instance_only, bool accept_overl
   }
 
   VALUE name = parse_method_name(state, &name_range);
-  VALUE method_types = rb_ary_new();
-  VALUE overload = Qfalse;
+  VALUE overloads = rb_ary_new();
+  VALUE overloading = Qfalse;
 
   if (state->next_token.type == pDOT && RB_SYM2ID(name) == rb_intern("self?")) {
     raise_syntax_error(
@@ -1507,23 +1542,33 @@ VALUE parse_member_def(parserstate *state, bool instance_only, bool accept_overl
 
   bool loop = true;
   while (loop) {
+    VALUE annotations = rb_ary_new();
+    position overload_annot_pos = NullPosition;
+
+    if (state->next_token.type == tANNOTATION) {
+      parse_annotations(state, annotations, &overload_annot_pos);
+    }
+
     switch (state->next_token.type) {
     case pLPAREN:
     case pARROW:
     case pLBRACE:
     case pLBRACKET:
     case pQUESTION:
-      rb_ary_push(method_types, parse_method_type(state));
-      member_range.end = state->current_token.range.end;
-      break;
+      {
+        VALUE method_type = parse_method_type(state);
+        rb_ary_push(overloads, rbs_ast_members_method_definition_overload(annotations, method_type));
+        member_range.end = state->current_token.range.end;
+        break;
+      }
 
     case pDOT3:
       if (accept_overload) {
-        overload = Qtrue;
+        overloading = Qtrue;
         parser_advance(state);
         loop = false;
-        overload_range = state->current_token.range;
-        member_range.end = overload_range.end;
+        overloading_range = state->current_token.range;
+        member_range.end = overloading_range.end;
         break;
       } else {
         raise_syntax_error(
@@ -1570,17 +1615,17 @@ VALUE parse_member_def(parserstate *state, bool instance_only, bool accept_overl
   rbs_loc_add_required_child(loc, rb_intern("keyword"), keyword_range);
   rbs_loc_add_required_child(loc, rb_intern("name"), name_range);
   rbs_loc_add_optional_child(loc, rb_intern("kind"), kind_range);
-  rbs_loc_add_optional_child(loc, rb_intern("overload"), overload_range);
+  rbs_loc_add_optional_child(loc, rb_intern("overloading"), overloading_range);
   rbs_loc_add_optional_child(loc, rb_intern("visibility"), visibility_range);
 
   return rbs_ast_members_method_definition(
     name,
     k,
-    method_types,
+    overloads,
     annotations,
     location,
     comment,
-    overload,
+    overloading,
     visibility
   );
 }
@@ -2223,13 +2268,11 @@ VALUE parse_module_members(parserstate *state) {
 }
 
 /*
-  module_decl ::= {`module`} module_name module_type_params module_members <kEND>
-                | {`module`} module_name module_type_params `:` module_self_types module_members <kEND>
+  module_decl ::= {module_name} module_type_params module_members <kEND>
+                | {module_name} module_name module_type_params `:` module_self_types module_members <kEND>
 */
-VALUE parse_module_decl(parserstate *state, position comment_pos, VALUE annotations) {
+VALUE parse_module_decl0(parserstate *state, range keyword_range, VALUE module_name, range name_range, VALUE comment, VALUE annotations) {
   range decl_range;
-  range keyword_range;
-  range name_range;
   range end_range;
   range type_params_range;
   range colon_range;
@@ -2237,15 +2280,8 @@ VALUE parse_module_decl(parserstate *state, position comment_pos, VALUE annotati
 
   parser_push_typevar_table(state, true);
 
-  position start = state->current_token.range.start;
-  comment_pos = nonnull_pos_or(comment_pos, start);
-  VALUE comment = get_comment(state, comment_pos.line);
+  decl_range.start = keyword_range.start;
 
-  keyword_range = state->current_token.range;
-  decl_range.start = state->current_token.range.start;
-
-  parser_advance(state);
-  VALUE module_name = parse_type_name(state, CLASS_NAME, &name_range);
   VALUE type_params = parse_type_params(state, &type_params_range, true);
   VALUE self_types = rb_ary_new();
 
@@ -2288,6 +2324,46 @@ VALUE parse_module_decl(parserstate *state, position comment_pos, VALUE annotati
   );
 }
 
+/*
+  module_decl ::= {`module`} module_name `=` old_module_name <kEND>
+                | {`module`} module_name module_decl0 <kEND>
+
+*/
+VALUE parse_module_decl(parserstate *state, position comment_pos, VALUE annotations) {
+  range keyword_range = state->current_token.range;
+  range module_name_range;
+
+  comment_pos = nonnull_pos_or(comment_pos, state->current_token.range.start);
+  VALUE comment = get_comment(state, comment_pos.line);
+
+  parser_advance(state);
+  VALUE module_name = parse_type_name(state, CLASS_NAME, &module_name_range);
+
+  if (state->next_token.type == pEQ) {
+    range eq_range = state->next_token.range;
+    parser_advance(state);
+    parser_advance(state);
+
+    range old_name_range;
+    VALUE old_name = parse_type_name(state, CLASS_NAME, &old_name_range);
+
+    range decl_range;
+    decl_range.start = keyword_range.start;
+    decl_range.end = old_name_range.end;
+
+    VALUE location = rbs_new_location(state->buffer, decl_range);
+    rbs_loc *loc = rbs_check_location(location);
+    rbs_loc_add_required_child(loc, rb_intern("keyword"), keyword_range);
+    rbs_loc_add_required_child(loc, rb_intern("new_name"), module_name_range);
+    rbs_loc_add_required_child(loc, rb_intern("eq"), eq_range);
+    rbs_loc_add_optional_child(loc, rb_intern("old_name"), old_name_range);
+
+    return rbs_ast_decl_module_alias(module_name, old_name, location, comment);
+  } else {
+    return parse_module_decl0(state, keyword_range, module_name, module_name_range, comment, annotations);
+  }
+}
+
 /*
   class_decl_super ::= {} `<` <class_instance_name>
                      | {<>}
@@ -2309,7 +2385,7 @@ VALUE parse_class_decl_super(parserstate *state, range *lt_range) {
     args = rb_ary_new();
     class_instance_name(state, CLASS_NAME, &name, args, &name_range, &args_range);
 
-    super_range.end = args_range.end;
+    super_range.end = state->current_token.range.end;
 
     location = rbs_new_location(state->buffer, super_range);
     loc = rbs_check_location(location);
@@ -2324,35 +2400,25 @@ VALUE parse_class_decl_super(parserstate *state, range *lt_range) {
 }
 
 /*
-  class_decl ::= {`class`} class_name type_params class_decl_super class_members <`end`>
+  class_decl ::= {class_name} type_params class_decl_super class_members <`end`>
 */
-VALUE parse_class_decl(parserstate *state, position comment_pos, VALUE annotations) {
+VALUE parse_class_decl0(parserstate *state, range keyword_range, VALUE name, range name_range, VALUE comment, VALUE annotations) {
   range decl_range;
-  range keyword_range;
-  range name_range;
   range end_range;
   range type_params_range;
   range lt_range;
 
-  VALUE name;
   VALUE type_params;
   VALUE super;
   VALUE members;
-  VALUE comment;
   VALUE location;
 
   rbs_loc *loc;
 
   parser_push_typevar_table(state, true);
 
-  decl_range.start = state->current_token.range.start;
-  keyword_range = state->current_token.range;
+  decl_range.start = keyword_range.start;
 
-  comment_pos = nonnull_pos_or(comment_pos, decl_range.start);
-  comment = get_comment(state, comment_pos.line);
-
-  parser_advance(state);
-  name = parse_type_name(state, CLASS_NAME, &name_range);
   type_params = parse_type_params(state, &type_params_range, true);
   super = parse_class_decl_super(state, &lt_range);
   members = parse_module_members(state);
@@ -2382,6 +2448,45 @@ VALUE parse_class_decl(parserstate *state, position comment_pos, VALUE annotatio
   );
 }
 
+/*
+  class_decl ::= {`class`} class_name `=` <class_name>
+               | {`class`} class_name <class_decl0>
+*/
+VALUE parse_class_decl(parserstate *state, position comment_pos, VALUE annotations) {
+  range keyword_range = state->current_token.range;
+  range class_name_range;
+
+  comment_pos = nonnull_pos_or(comment_pos, state->current_token.range.start);
+  VALUE comment = get_comment(state, comment_pos.line);
+
+  parser_advance(state);
+  VALUE class_name = parse_type_name(state, CLASS_NAME, &class_name_range);
+
+  if (state->next_token.type == pEQ) {
+    range eq_range = state->next_token.range;
+    parser_advance(state);
+    parser_advance(state);
+
+    range old_name_range;
+    VALUE old_name = parse_type_name(state, CLASS_NAME, &old_name_range);
+
+    range decl_range;
+    decl_range.start = keyword_range.start;
+    decl_range.end = old_name_range.end;
+
+    VALUE location = rbs_new_location(state->buffer, decl_range);
+    rbs_loc *loc = rbs_check_location(location);
+    rbs_loc_add_required_child(loc, rb_intern("keyword"), keyword_range);
+    rbs_loc_add_required_child(loc, rb_intern("new_name"), class_name_range);
+    rbs_loc_add_required_child(loc, rb_intern("eq"), eq_range);
+    rbs_loc_add_optional_child(loc, rb_intern("old_name"), old_name_range);
+
+    return rbs_ast_decl_class_alias(class_name, old_name, location, comment);
+  } else {
+    return parse_class_decl0(state, keyword_range, class_name, class_name_range, comment, annotations);
+  }
+}
+
 /*
   nested_decl ::= {<const_decl>}
                 | {<class_decl>}
@@ -2457,23 +2562,192 @@ VALUE parse_decl(parserstate *state) {
   }
 }
 
+/*
+  namespace ::= {} (`::`)? (`tUIDENT` `::`)* `tUIDENT` <`::`>
+              | {} <>                                            (empty -- returns empty namespace)
+*/
+VALUE parse_namespace(parserstate *state, range *rg) {
+  bool is_absolute = false;
+
+  if (state->next_token.type == pCOLON2) {
+    rg->start = state->next_token.range.start;
+    rg->end = state->next_token.range.end;
+    is_absolute = true;
+
+    parser_advance(state);
+  }
+
+  VALUE path = rb_ary_new();
+
+  while (true) {
+    if (state->next_token.type == tUIDENT && state->next_token2.type == pCOLON2) {
+      rb_ary_push(path, ID2SYM(INTERN_TOKEN(state, state->next_token)));
+      if (null_position_p(rg->start)) {
+        rg->start = state->next_token.range.start;
+      }
+      rg->end = state->next_token2.range.end;
+      parser_advance(state);
+      parser_advance(state);
+    } else {
+      break;
+    }
+  }
+
+  return rbs_namespace(path, is_absolute ? Qtrue : Qfalse);
+}
+
+/*
+  use_clauses ::= {} use_clause `,` ... `,` <use_clause>
+
+  use_clause ::= {} namespace <tUIDENT>
+               | {} namespace tUIDENT `as` <tUIDENT>
+               | {} namespace <tSTAR>
+*/
+void parse_use_clauses(parserstate *state, VALUE clauses) {
+  while (true) {
+    range namespace_range = NULL_RANGE;
+    VALUE namespace = parse_namespace(state, &namespace_range);
+
+    range clause_range = namespace_range;
+
+    switch (state->next_token.type)
+    {
+      case tLIDENT:
+      case tULIDENT:
+      case tUIDENT: {
+        parser_advance(state);
+
+        enum TokenType ident_type = state->current_token.type;
+
+        range type_name_range;
+        if (null_range_p(namespace_range)) {
+          type_name_range = state->current_token.range;
+        } else {
+          type_name_range.start = namespace_range.start;
+          type_name_range.end = state->current_token.range.end;
+        }
+        clause_range = type_name_range;
+
+        VALUE type_name = rbs_type_name(namespace, ID2SYM(INTERN_TOKEN(state, state->current_token)));
+
+        range keyword_range = NULL_RANGE;
+        range new_name_range = NULL_RANGE;
+
+        VALUE new_name = Qnil;
+        if (state->next_token.type == kAS) {
+          parser_advance(state);
+          keyword_range = state->current_token.range;
+
+          if (ident_type == tUIDENT) parser_advance_assert(state, tUIDENT);
+          if (ident_type == tLIDENT) parser_advance_assert(state, tLIDENT);
+          if (ident_type == tULIDENT) parser_advance_assert(state, tULIDENT);
+
+          new_name = ID2SYM(INTERN_TOKEN(state, state->current_token));
+          new_name_range = state->current_token.range;
+          clause_range.end = new_name_range.end;
+        }
+
+        VALUE location = rbs_new_location(state->buffer, clause_range);
+        rbs_loc *loc = rbs_check_location(location);
+        rbs_loc_add_required_child(loc, rb_intern("type_name"), state->current_token.range);
+        if (!null_range_p(keyword_range)) {
+          rbs_loc_add_optional_child(loc, rb_intern("keyword"), keyword_range);
+        }
+        if (!null_range_p(new_name_range)) {
+          rbs_loc_add_optional_child(loc, rb_intern("new_name"), new_name_range);
+        }
+
+        rb_ary_push(clauses, rbs_ast_directives_use_single_clause(type_name, new_name, location));
+
+        break;
+      }
+      case pSTAR:
+      {
+        parser_advance(state);
+
+        range star_range = state->current_token.range;
+        clause_range.end = star_range.end;
+
+        VALUE location = rbs_new_location(state->buffer, clause_range);
+        rbs_loc *loc = rbs_check_location(location);
+        rbs_loc_add_required_child(loc, rb_intern("namespace"), namespace_range);
+        rbs_loc_add_required_child(loc, rb_intern("star"), star_range);
+
+        rb_ary_push(clauses, rbs_ast_directives_use_wildcard_clause(namespace, location));
+
+        break;
+      }
+      default:
+        raise_syntax_error(
+          state,
+          state->next_token,
+          "use clause is expected"
+        );
+    }
+
+    if (state->next_token.type == pCOMMA) {
+      parser_advance(state);
+    } else {
+      break;
+    }
+  }
+
+  return;
+}
+
+/*
+  use_directive ::= {} `use` <clauses>
+ */
+VALUE parse_use_directive(parserstate *state) {
+  if (state->next_token.type == kUSE) {
+    parser_advance(state);
+
+    range keyword_range = state->current_token.range;
+
+    VALUE clauses = rb_ary_new();
+    parse_use_clauses(state, clauses);
+
+    range directive_range = keyword_range;
+    directive_range.end = state->current_token.range.end;
+
+    VALUE location = rbs_new_location(state->buffer, directive_range);
+    rbs_loc *loc = rbs_check_location(location);
+    rbs_loc_add_required_child(loc, rb_intern("keyword"), keyword_range);
+
+    return rbs_ast_directives_use(clauses, location);
+  } else {
+    return Qnil;
+  }
+}
+
 VALUE parse_signature(parserstate *state) {
+  VALUE dirs = rb_ary_new();
   VALUE decls = rb_ary_new();
 
+  while (state->next_token.type == kUSE) {
+    rb_ary_push(dirs, parse_use_directive(state));
+  }
+
   while (state->next_token.type != pEOF) {
     rb_ary_push(decls, parse_decl(state));
   }
 
-  return decls;
+  VALUE ret = rb_ary_new();
+  rb_ary_push(ret, dirs);
+  rb_ary_push(ret, decls);
+  return ret;
 }
 
 static VALUE
-rbsparser_parse_type(VALUE self, VALUE buffer, VALUE line, VALUE column, VALUE variables)
+rbsparser_parse_type(VALUE self, VALUE buffer, VALUE start_pos, VALUE end_pos, VALUE variables)
 {
-  parserstate *parser = alloc_parser(buffer, FIX2INT(line), FIX2INT(column), variables);
+  parserstate *parser = alloc_parser(buffer, FIX2INT(start_pos), FIX2INT(end_pos), variables);
+
+  if (parser->next_token.type == pEOF) {
+    return Qnil;
+  }
 
   VALUE type = parse_type(parser);
-  parser_advance_assert(parser, pEOF);
 
   free_parser(parser);
 
@@ -2481,28 +2755,34 @@ rbsparser_parse_type(VALUE self, VALUE buffer, VALUE line, VALUE column, VALUE v
 }
 
 static VALUE
-rbsparser_parse_method_type(VALUE self, VALUE buffer, VALUE line, VALUE column, VALUE variables)
+rbsparser_parse_method_type(VALUE self, VALUE buffer, VALUE start_pos, VALUE end_pos, VALUE variables)
 {
-  parserstate *parser = alloc_parser(buffer, FIX2INT(line), FIX2INT(column), variables);
+  parserstate *parser = alloc_parser(buffer, FIX2INT(start_pos), FIX2INT(end_pos), variables);
+
+  if (parser->next_token.type == pEOF) {
+    return Qnil;
+  }
+
   VALUE method_type = parse_method_type(parser);
-  free(parser);
+
+  free_parser(parser);
 
   return method_type;
 }
 
 static VALUE
-rbsparser_parse_signature(VALUE self, VALUE buffer, VALUE line, VALUE column)
+rbsparser_parse_signature(VALUE self, VALUE buffer, VALUE end_pos)
 {
-  parserstate *parser = alloc_parser(buffer, FIX2INT(line), FIX2INT(column), Qnil);
-  VALUE signature = parse_signature(parser);
+  parserstate *parser = alloc_parser(buffer, 0, FIX2INT(end_pos), Qnil);
+  VALUE pair = parse_signature(parser);
   free_parser(parser);
 
-  return signature;
+  return pair;
 }
 
 void rbs__init_parser(void) {
   RBS_Parser = rb_define_class_under(RBS, "Parser", rb_cObject);
   rb_define_singleton_method(RBS_Parser, "_parse_type", rbsparser_parse_type, 4);
   rb_define_singleton_method(RBS_Parser, "_parse_method_type", rbsparser_parse_method_type, 4);
-  rb_define_singleton_method(RBS_Parser, "_parse_signature", rbsparser_parse_signature, 3);
+  rb_define_singleton_method(RBS_Parser, "_parse_signature", rbsparser_parse_signature, 2);
 }
