@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require "open3"
 require "optparse"
 require "shellwords"
@@ -17,7 +19,7 @@ module RBS
 
         @libs = []
         @dirs = []
-        @config_path = Collection::Config::PATH
+        @config_path = Collection::Config.find_config_path || Collection::Config::PATH
       end
 
       def loader
@@ -27,7 +29,12 @@ module RBS
         end
 
         loader = EnvironmentLoader.new(core_root: core_root, repository: repository)
-        lock = config_path&.then { |p| Collection::Config.lockfile_of(p) }
+        if config_path
+          lock_path = Collection::Config.to_lockfile_path(config_path)
+          if lock_path.file?
+            lock = Collection::Config::Lockfile.from_lockfile(lockfile_path: lock_path, data: YAML.load_file(lock_path.to_s))
+          end
+        end
         loader.add_collection(lock) if lock
 
         dirs.each do |dir|
@@ -56,11 +63,11 @@ module RBS
           self.core_root = nil
         end
 
-        opts.on('--collection PATH', "File path of collection configration (default: #{Collection::Config::PATH})") do |path|
+        opts.on('--collection PATH', "File path of collection configuration (default: #{@config_path})") do |path|
           self.config_path = Pathname(path).expand_path
         end
 
-        opts.on('--no-collection', 'Ignore collection configration') do
+        opts.on('--no-collection', 'Ignore collection configuration') do
           self.config_path = nil
         end
 
@@ -74,6 +81,7 @@ module RBS
 
     attr_reader :stdout
     attr_reader :stderr
+    attr_reader :original_args
 
     def initialize(stdout:, stderr:)
       @stdout = stdout
@@ -100,6 +108,8 @@ module RBS
     end
 
     def run(args)
+      @original_args = args.dup
+
       options = LibraryOptions.new
 
       opts = OptionParser.new
@@ -220,6 +230,19 @@ EOB
             end
           end
         end
+
+        env.class_alias_decls.each do |name, entry|
+          case entry
+          when Environment::ModuleAliasEntry
+            if list.include?(:module)
+              stdout.puts "#{name} (module alias)"
+            end
+          when Environment::ClassAliasEntry
+            if list.include?(:class)
+              stdout.puts "#{name} (class alias)"
+            end
+          end
+        end
       end
 
       if list.include?(:interface)
@@ -250,6 +273,11 @@ EOU
         opts.on("--singleton", "Ancestors of singleton of the given type_name") { kind = :singleton }
       end.order!(args)
 
+      unless args.size == 1
+        stdout.puts "Expected one argument."
+        return
+      end
+
       loader = options.loader()
 
       env = Environment.from_loader(loader).resolve_type_names
@@ -257,7 +285,10 @@ EOU
       builder = DefinitionBuilder::AncestorBuilder.new(env: env)
       type_name = TypeName(args[0]).absolute!
 
-      if env.class_decls.key?(type_name)
+      case env.constant_entry(type_name)
+      when Environment::ClassEntry, Environment::ModuleEntry, Environment::ClassAliasEntry, Environment::ModuleAliasEntry
+        type_name = env.normalize_module_name(type_name)
+
         ancestors = case kind
                     when :instance
                       builder.instance_ancestors(type_name)
@@ -319,7 +350,7 @@ EOU
       builder = DefinitionBuilder.new(env: env)
       type_name = TypeName(args[0]).absolute!
 
-      if env.class_decls.key?(type_name)
+      if env.module_name?(type_name)
         definition = case kind
                      when :instance
                        builder.build_instance(type_name)
@@ -373,7 +404,7 @@ EOU
       type_name = TypeName(args[0]).absolute!
       method_name = args[1].to_sym
 
-      unless env.class_decls.key?(type_name)
+      unless env.module_name?(type_name)
         stdout.puts "Cannot find class: #{type_name}"
         return
       end
@@ -428,15 +459,15 @@ EOU
       env = Environment.from_loader(loader).resolve_type_names
 
       builder = DefinitionBuilder.new(env: env)
-      validator = Validator.new(env: env, resolver: TypeNameResolver.from_env(env))
+      validator = Validator.new(env: env, resolver: Resolver::TypeNameResolver.new(env))
 
       env.class_decls.each do |name, decl|
         stdout.puts "Validating class/module definition: `#{name}`..."
         builder.build_instance(name).each_type do |type|
-          validator.validate_type type, context: [Namespace.root]
+          validator.validate_type type, context: nil
         end
         builder.build_singleton(name).each_type do |type|
-          validator.validate_type type, context: [Namespace.root]
+          validator.validate_type type, context: nil
         end
 
         d = decl.primary.decl
@@ -457,10 +488,15 @@ EOU
         end
       end
 
+      env.class_alias_decls.each do |name, entry|
+        stdout.puts "Validating class/module alias definition: `#{name}`..."
+        validator.validate_class_alias(entry: entry)
+      end
+
       env.interface_decls.each do |name, decl|
         stdout.puts "Validating interface: `#{name}`..."
         builder.build_interface(name).each_type do |type|
-          validator.validate_type type, context: [Namespace.root]
+          validator.validate_type type, context: nil
         end
 
         validator.validate_type_params(
@@ -485,13 +521,13 @@ EOU
 
       env.global_decls.each do |name, global|
         stdout.puts "Validating global: `#{name}`..."
-        validator.validate_type global.decl.type, context: [Namespace.root]
+        validator.validate_type global.decl.type, context: nil
       end
 
-      env.alias_decls.each do |name, decl|
+      env.type_alias_decls.each do |name, decl|
         stdout.puts "Validating alias: `#{name}`..."
         builder.expand_alias1(name).tap do |type|
-          validator.validate_type type, context: [Namespace.root]
+          validator.validate_type type, context: nil
         end
         validator.validate_type_alias(entry: decl)
       end
@@ -527,14 +563,26 @@ EOU
       env = Environment.from_loader(loader).resolve_type_names
 
       builder = DefinitionBuilder.new(env: env)
-      table = ConstantTable.new(builder: builder)
+      resolver = Resolver::ConstantResolver.new(builder: builder)
 
-      namespace = context ? Namespace.parse(context).absolute! : Namespace.root
-      stdout.puts "Context: #{namespace}"
-      name = Namespace.parse(args[0]).to_type_name
-      stdout.puts "Constant name: #{name}"
+      resolver_context = context ? [nil, TypeName(context).absolute!] : nil #: Resolver::context
+      stdout.puts "Context: #{context}"
+      const_name = TypeName(args[0])
+      stdout.puts "Constant name: #{const_name}"
 
-      constant = table.resolve_constant_reference(name, context: namespace.ascend.to_a)
+      if const_name.absolute?
+        constant = resolver.table.constant(const_name)
+      else
+        head, *components = const_name.to_namespace.path
+        head or raise
+
+        constant = resolver.resolve(head, context: resolver_context)
+        constant = components.inject(constant) do |const, component|
+          if const
+            resolver.resolve_child(const.name, component)
+          end
+        end
+      end
 
       if constant
         stdout.puts " => #{constant.name}: #{constant.type}"
@@ -667,6 +715,8 @@ EOU
       output_dir = nil
       # @type var base_dir: Pathname?
       base_dir = nil
+      # @type var force: bool
+      force = false
 
       opts = OptionParser.new
       opts.banner = <<EOU
@@ -696,6 +746,10 @@ EOU
         base_dir = Pathname(path)
       end
 
+      opts.on("--force", "Overwrite existing RBS files") do
+        force = true
+      end
+
       opts.parse!(args)
 
       unless has_parser?
@@ -714,12 +768,17 @@ EOU
           Prototype::RBI.new()
         when "rb"
           Prototype::RB.new()
+        else
+          raise
         end
       end
 
       input_paths = args.map {|arg| Pathname(arg) }
 
       if output_dir
+        # @type var skip_paths: Array[Pathname]
+        skip_paths = []
+
         # batch mode
         input_paths.each do |path|
           stdout.puts "Processing `#{path}`..."
@@ -762,7 +821,17 @@ EOU
             parser = new_parser[]
             parser.parse file_path.read()
 
-            stdout.puts "  Writing RBS to `#{output_path}`..."
+            if output_path.file?
+              if force
+                stdout.puts "    - Writing RBS to existing file `#{output_path}`..."
+              else
+                stdout.puts "    - Skipping existing file `#{output_path}`..."
+                skip_paths << file_path
+                next
+              end
+            else
+              stdout.puts "    - Writing RBS to `#{output_path}`..."
+            end
 
             (output_path.parent).mkpath
             output_path.open("w") do |io|
@@ -771,6 +840,19 @@ EOU
             end
           end
         end
+
+        unless skip_paths.empty?
+          stdout.puts
+          stdout.puts ">>>> Skipped existing #{skip_paths.size} files. Use `--force` option to update the files."
+          command = original_args.take(original_args.size - input_paths.size)
+
+          skip_paths.take(10).each do |path|
+            stdout.puts "  #{defined?(Bundler) ? "bundle exec " : ""}rbs #{Shellwords.join(command)} --force #{Shellwords.escape(path.to_s)}"
+          end
+          if skip_paths.size > 10
+            stdout.puts "  ..."
+          end
+        end
       else
         # file mode
         parser = new_parser[]
@@ -973,7 +1055,7 @@ EOB
       env_hash = {
         'RUBYOPT' => "#{ENV['RUBYOPT']} -rrbs/test/setup",
         'RBS_TEST_OPT' => test_opt(options),
-        'RBS_TEST_LOGLEVEL' => RBS.logger_level,
+        'RBS_TEST_LOGLEVEL' => %w(DEBUG INFO WARN ERROR FATAL)[RBS.logger_level || 5] || "UNKNOWN",
         'RBS_TEST_SAMPLE_SIZE' => sample_size,
         'RBS_TEST_DOUBLE_SUITE' => double_suite,
         'RBS_TEST_UNCHECKED_CLASSES' => (unchecked_classes.join(',') unless unchecked_classes.empty?),
@@ -999,12 +1081,12 @@ EOB
       case args[0]
       when 'install'
         unless params[:frozen]
-          Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: Pathname('./Gemfile.lock'))
+          Collection::Config.generate_lockfile(config_path: config_path, definition: Bundler.definition)
         end
         Collection::Installer.new(lockfile_path: lock_path, stdout: stdout).install_from_lockfile
       when 'update'
         # TODO: Be aware of argv to update only specified gem
-        Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: Pathname('./Gemfile.lock'), with_lockfile: false)
+        Collection::Config.generate_lockfile(config_path: config_path, definition: Bundler.definition, with_lockfile: false)
         Collection::Installer.new(lockfile_path: lock_path, stdout: stdout).install_from_lockfile
       when 'init'
         if config_path.exist?
@@ -1015,11 +1097,16 @@ EOB
         config_path.write(<<~'YAML')
           # Download sources
           sources:
-            - name: ruby/gem_rbs_collection
+            - type: git
+              name: ruby/gem_rbs_collection
               remote: https://github.com/ruby/gem_rbs_collection.git
               revision: main
               repo_dir: gems
 
+          # You can specify local directories as sources also.
+          # - type: local
+          #   path: path/to/your/local/repository
+
           # A directory to install the downloaded RBSs
           path: .gem_rbs_collection
 
@@ -1053,7 +1140,7 @@ EOB
 
           Examples:
 
-            # Initialize the configration file
+            # Initialize the configuration file
             $ rbs collection init
 
             # Generate the lock file and install RBSs from the lock file
@@ -1061,6 +1148,8 @@ EOB
 
             # Update the RBSs
             $ rbs collection update
+
+          Options:
         HELP
         opts.on('--frozen') if args[0] == 'install'
       end
