@@ -47,7 +47,7 @@ class RBS::CliTest < Test::Unit::TestCase
 
   def test_ast
     with_cli do |cli|
-      cli.run(%w(-r set ast))
+      cli.run(%w(ast))
 
       # Outputs a JSON
       JSON.parse stdout.string
@@ -90,11 +90,29 @@ class RBS::CliTest < Test::Unit::TestCase
       refute_match %r{^::Kernel \(module\)$}, stdout.string
       assert_match %r{^::_Each \(interface\)$}, stdout.string
     end
+
+    Dir.mktmpdir do |dir|
+      dir = Pathname(dir)
+      dir.join('alias.rbs').write(<<~RBS)
+        class Foo = String
+
+        module Bar = Kernel
+      RBS
+
+      Dir.chdir(dir) do
+        with_cli do |cli|
+          cli.run(%w(-I. list))
+
+          assert_match %r{^::Foo \(class alias\)$}, stdout.string
+          assert_match %r{^::Bar \(module alias\)$}, stdout.string
+        end
+      end
+    end
   end
 
   def test_ancestors
     with_cli do |cli|
-      cli.run(%w(-r set ancestors ::Set))
+      cli.run(%w(ancestors ::Set))
       assert_equal <<-EOF, stdout.string
 ::Set[A]
 ::Enumerable[A]
@@ -105,7 +123,7 @@ class RBS::CliTest < Test::Unit::TestCase
     end
 
     with_cli do |cli|
-      cli.run(%w(-r set ancestors --instance ::Set))
+      cli.run(%w(ancestors --instance ::Set))
       assert_equal <<-EOF, stdout.string
 ::Set[A]
 ::Enumerable[A]
@@ -116,7 +134,7 @@ class RBS::CliTest < Test::Unit::TestCase
     end
 
     with_cli do |cli|
-      cli.run(%w(-r set ancestors --singleton ::Set))
+      cli.run(%w(ancestors --singleton ::Set))
       assert_equal <<-EOF, stdout.string
 singleton(::Set)
 singleton(::Object)
@@ -128,19 +146,64 @@ singleton(::BasicObject)
 ::BasicObject
       EOF
     end
+
+    Dir.mktmpdir do |dir|
+      dir = Pathname(dir)
+      dir.join('alias.rbs').write(<<~RBS)
+        class Foo = String
+
+        class Bar
+        end
+      RBS
+
+      Dir.chdir(dir) do
+        with_cli do |cli|
+          cli.run(%w(-I. ancestors ::Foo))
+
+          assert_equal <<~EOF, stdout.string
+            ::String
+            ::Comparable
+            ::Object
+            ::Kernel
+            ::BasicObject
+          EOF
+        end
+      end
+    end
   end
 
   def test_methods
     with_cli do |cli|
-      cli.run(%w(-r set methods ::Set))
-      cli.run(%w(-r set methods --instance ::Set))
-      cli.run(%w(-r set methods --singleton ::Set))
+      cli.run(%w(methods ::Set))
+      cli.run(%w(methods --instance ::Set))
+      cli.run(%w(methods --singleton ::Set))
+    end
+
+    Dir.mktmpdir do |dir|
+      dir = Pathname(dir)
+      dir.join('alias.rbs').write(<<~RBS)
+        class Foo = String
+
+        module Bar = Kernel
+      RBS
+
+      Dir.chdir(dir) do
+        with_cli do |cli|
+          cli.run(%w(-I. methods ::Foo))
+          assert_match %r{^puts \(private\)$}, stdout.string
+        end
+
+        with_cli do |cli|
+          cli.run(%w(-I. methods --singleton ::Bar))
+          assert_match %r{^puts \(public\)$}, stdout.string
+        end
+      end
     end
   end
 
   def test_method
     with_cli do |cli|
-      cli.run(%w(-r set method ::Object yield_self))
+      cli.run(%w(method ::Object yield_self))
       assert_equal <<-EOF, stdout.string
 ::Object#yield_self
   defined_in: ::Object
@@ -151,11 +214,33 @@ singleton(::BasicObject)
     | () -> ::Enumerator[self, untyped]
       EOF
     end
+
+    Dir.mktmpdir do |dir|
+      dir = Pathname(dir)
+      dir.join('alias.rbs').write(<<~RBS)
+        class Foo = String
+
+        module Bar = Kernel
+      RBS
+
+      Dir.chdir(dir) do
+        with_cli do |cli|
+          cli.run(%w(-I. method ::Foo puts))
+          assert_match %r{^::Foo#puts$}, stdout.string
+        end
+
+        with_cli do |cli|
+          cli.run(%w(-I. method --singleton ::Bar puts))
+          assert_match %r{^::Bar\.puts$}, stdout.string
+        end
+      end
+    end
+
   end
 
   def test_validate
     with_cli do |cli|
-      cli.run(%w(-r set validate))
+      cli.run(%w(validate))
       assert_match(/Validating/, stdout.string)
     end
 
@@ -317,22 +402,22 @@ singleton(::BasicObject)
 
   def test_constant
     with_cli do |cli|
-      cli.run(%w(-r set constant Pathname))
-      cli.run(%w(-r set constant --context File IO))
+      cli.run(%w(constant Pathname))
+      cli.run(%w(constant --context File IO))
     end
   end
 
   def test_version
     with_cli do |cli|
-      cli.run(%w(-r set version))
+      cli.run(%w(version))
     end
   end
 
   def test_paths
     with_cli do |cli|
-      cli.run(%w(-r set -I sig/test paths))
+      cli.run(%w(-r pathname -I sig/test paths))
       assert_match %r{/rbs/core \(dir, core\)$}, stdout.string
-      assert_match %r{/rbs/stdlib/set/0 \(dir, library, name=set\)$}, stdout.string
+      assert_match %r{/rbs/stdlib/pathname/0 \(dir, library, name=pathname\)$}, stdout.string
       assert_match %r{^sig/test \(absent\)$}, stdout.string
     end
   end
