@@ -18,6 +18,13 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     path: /path/to/somewhere
   YAML
 
+  GEMFILE = <<~RUBY
+    source 'https://rubygems.org'
+
+    gem 'ast'
+    gem 'rainbow'
+  RUBY
+
   GEMFILE_LOCK = <<~YAML
     GEM
       remote: https://rubygems.org/
@@ -40,16 +47,20 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write GEMFILE
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
 
-      assert_config <<~YAML, io.string
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
+
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
@@ -92,18 +103,21 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
 
           path: /path/to/somewhere
         YAML
+        gemfile_path = tmpdir / 'Gemfile'
+        gemfile_path.write GEMFILE
         gemfile_lock_path = tmpdir / 'Gemfile.lock'
         gemfile_lock_path.write GEMFILE_LOCK
 
-        config = Dir.chdir(tmpdir) do
-          RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
+        _config, lockfile = Dir.chdir(tmpdir) do
+          definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+          RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
         end
-        io = StringIO.new
-        config.dump_to(io)
+        string = YAML.dump(lockfile.to_lockfile)
 
-        assert_config <<~YAML, io.string
+        assert_config <<~YAML, string
           sources:
-            - name: ruby/gem_rbs_collection
+            - type: git
+              name: ruby/gem_rbs_collection
               remote: #{remote}
               revision: b4d3b346d9657543099a35a1fd20347e75b8c523
               repo_dir: gems
@@ -135,12 +149,15 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write GEMFILE
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write GEMFILE_LOCK
 
-      lockfile = <<~YAML
+      lockfile_yaml = <<~YAML
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
@@ -164,13 +181,14 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
               repo_dir: gems
               type: git
       YAML
-      tmpdir.join('rbs_collection.lock.yaml').write lockfile
+      tmpdir.join('rbs_collection.lock.yaml').write lockfile_yaml
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
 
-      assert_config lockfile, io.string
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
+
+      assert_config lockfile_yaml, string
     end
   end
 
@@ -184,79 +202,25 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
           - name: rainbow
             ignore: false
       YAML
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write GEMFILE
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
 
-      assert_config <<~YAML, io.string
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
         path: "/path/to/somewhere"
         gemfile_lock_path: 'Gemfile.lock'
         gems:
-          - name: rainbow
-            ignore: false
-            version: "3.0"
-            source:
-              name: ruby/gem_rbs_collection
-              remote: https://github.com/ruby/gem_rbs_collection.git
-              revision: cde6057e7546843ace6420c5783dd945c6ccda54
-              repo_dir: gems
-              type: git
-      YAML
-    end
-  end
-
-  def test_generate_lock_from_collection_repository_specified
-    mktmpdir do |tmpdir|
-      config_path = tmpdir / 'rbs_collection.yaml'
-      config_path.write [CONFIG, <<~YAML].join("\n")
-        gems:
-          - name: ast
-          - name: rainbow
-      YAML
-      gemfile_lock_path = tmpdir / 'Gemfile.lock'
-      gemfile_lock_path.write <<~GEMFILE_LOCK
-        GEM
-          remote: https://rubygems.org/
-          specs:
-
-        PLATFORMS
-          x86_64-linux
-
-        DEPENDENCIES
-
-        BUNDLED WITH
-           2.2.0
-      GEMFILE_LOCK
-
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
-
-      assert_config <<~YAML, io.string
-        sources:
-          - name: ruby/gem_rbs_collection
-            remote: https://github.com/ruby/gem_rbs_collection.git
-            revision: cde6057e7546843ace6420c5783dd945c6ccda54
-            repo_dir: gems
-        path: "/path/to/somewhere"
-        gemfile_lock_path: 'Gemfile.lock'
-        gems:
-          - name: ast
-            version: "2.4"
-            source:
-              name: ruby/gem_rbs_collection
-              remote: https://github.com/ruby/gem_rbs_collection.git
-              revision: cde6057e7546843ace6420c5783dd945c6ccda54
-              repo_dir: gems
-              type: git
           - name: rainbow
             version: "3.0"
             source:
@@ -273,6 +237,12 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source "https://rubygems.org"
+
+        gem "activesupport"
+      GEMFILE
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write <<~GEMFILE_LOCK
         GEM
@@ -302,13 +272,14 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
            2.2.0
       GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
 
-      assert_config <<~YAML, io.string
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
@@ -323,6 +294,14 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
               revision: cde6057e7546843ace6420c5783dd945c6ccda54
               repo_dir: gems
               type: git
+          - name: date
+            version: "0"
+            source:
+              type: stdlib
+          - name: logger
+            version: "0"
+            source:
+              type: stdlib
           - name: minitest
             version: '0'
             source:
@@ -331,7 +310,7 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
             version: "0"
             source:
               type: stdlib
-          - name: date
+          - name: mutex_m
             version: "0"
             source:
               type: stdlib
@@ -339,14 +318,6 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
             version: "0"
             source:
               type: stdlib
-          - name: logger
-            version: "0"
-            source:
-              type: stdlib
-          - name: mutex_m
-            version: "0"
-            source:
-              type: stdlib
           - name: time
             version: "0"
             source:
@@ -359,6 +330,13 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source "https://rubygems.org"
+
+        gem "csv"
+      GEMFILE
+
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write <<~GEMFILE_LOCK
         GEM
@@ -376,13 +354,14 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
            2.2.0
       GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
 
-      assert_config <<~YAML, io.string
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
@@ -407,6 +386,12 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source "https://rubygems.org"
+
+        gem "rbs-amber"
+      GEMFILE
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write <<~GEMFILE_LOCK
         GEM
@@ -424,28 +409,29 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
            2.2.0
       GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
 
-      assert_config <<~YAML, io.string
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
         path: "/path/to/somewhere"
         gemfile_lock_path: 'Gemfile.lock'
         gems:
-          - name: rbs-amber
-            version: "1.0.0"
-            source:
-              type: rubygems
           - name: pathname
             version: "0"
             source:
               type: stdlib
-      YAML
+          - name: rbs-amber
+            version: "1.0.0"
+            source:
+              type: rubygems
+        YAML
     end
   end
 
@@ -453,6 +439,8 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     mktmpdir do |tmpdir|
       config_path = tmpdir / 'rbs_collection.yaml'
       config_path.write CONFIG
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write 'source "https://ruubygems.org"'
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write <<~GEMFILE_LOCK
         GEM
@@ -468,19 +456,19 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
            2.2.0
       GEMFILE_LOCK
 
-      config = RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path)
-      io = StringIO.new
-      config.dump_to(io)
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
 
-      assert_config <<~YAML, io.string
+      assert_config <<~YAML, string
         sources:
-          - name: ruby/gem_rbs_collection
+          - type: git
+            name: ruby/gem_rbs_collection
             remote: https://github.com/ruby/gem_rbs_collection.git
             revision: cde6057e7546843ace6420c5783dd945c6ccda54
             repo_dir: gems
         path: "/path/to/somewhere"
         gemfile_lock_path: 'Gemfile.lock'
-        gems: []
       YAML
     end
   end
@@ -493,11 +481,14 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
       lock_path.write CONFIG + "gemfile_lock_path: Gemfile.lock"
       gemfile_lock_path = tmpdir / 'Gemfile.lock'
       gemfile_lock_path.write GEMFILE_LOCK
+      gemfile_path2 = tmpdir / 'Gemfile.2'
+      gemfile_path2.write 'source "https://rubygems.org"'
       gemfile_lock_path2 = tmpdir / 'Gemfile.2.lock'
       gemfile_lock_path2.write GEMFILE_LOCK
 
       assert_raises(RBS::Collection::Config::LockfileGenerator::GemfileLockMismatchError) do
-        RBS::Collection::Config.generate_lockfile(config_path: config_path, gemfile_lock_path: gemfile_lock_path2)
+        definition = Bundler::Definition.build(gemfile_path2, gemfile_lock_path2, false)
+        RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
       end
     end
   end
@@ -517,6 +508,200 @@ class RBS::Collection::ConfigTest < Test::Unit::TestCase
     end
   end
 
+  def test_generate_lock_from_bundler_require_false
+    mktmpdir do |tmpdir|
+      config_path = tmpdir / 'rbs_collection.yaml'
+      config_path.write [CONFIG, <<~YAML].join("\n")
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: '.gem_rbs_collection'
+        gems: []
+      YAML
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source 'https://rubygems.org'
+
+        gem "activesupport", require: false
+        gem "ast"
+      GEMFILE
+      gemfile_lock_path = tmpdir / 'Gemfile.lock'
+      gemfile_lock_path.write <<~GEMFILE_LOCK
+        GEM
+          remote: https://rubygems.org/
+          specs:
+            activesupport (6.1.4.1)
+              concurrent-ruby (~> 1.0, >= 1.0.2)
+              i18n (>= 1.6, < 2)
+              minitest (>= 5.1)
+              tzinfo (~> 2.0)
+              zeitwerk (~> 2.3)
+            concurrent-ruby (1.1.9)
+            i18n (1.8.11)
+              concurrent-ruby (~> 1.0)
+            minitest (5.14.4)
+            tzinfo (2.0.4)
+              concurrent-ruby (~> 1.0)
+            zeitwerk (2.5.1)
+            ast (2.4.2)
+
+        PLATFORMS
+          x86_64-linux
+
+        DEPENDENCIES
+          activesupport
+          ast
+
+        BUNDLED WITH
+           2.2.0
+      GEMFILE_LOCK
+
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
+
+      assert_config <<~YAML, string
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: ".gem_rbs_collection"
+        gemfile_lock_path: 'Gemfile.lock'
+        gems:
+          - name: ast
+            version: "2.4"
+            source:
+              name: ruby/gem_rbs_collection
+              remote: https://github.com/ruby/gem_rbs_collection.git
+              revision: cde6057e7546843ace6420c5783dd945c6ccda54
+              repo_dir: gems
+              type: git
+      YAML
+    end
+  end
+
+  def test_generate_lock_install_indirect_dependency
+    mktmpdir do |tmpdir|
+      config_path = tmpdir / 'rbs_collection.yaml'
+      config_path.write [CONFIG, <<~YAML].join("\n")
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: '.gem_rbs_collection'
+        gems: []
+      YAML
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source 'https://rubygems.org'
+
+        gem "parser"
+      GEMFILE
+      gemfile_lock_path = tmpdir / 'Gemfile.lock'
+      gemfile_lock_path.write <<~GEMFILE_LOCK
+        GEM
+          remote: https://rubygems.org/
+          specs:
+            parser (3.2.0.0)
+              ast (~> 2.4.1)
+            ast (2.4.2)
+
+        PLATFORMS
+          x86_64-linux
+
+        DEPENDENCIES
+          parser
+
+        BUNDLED WITH
+           2.2.0
+      GEMFILE_LOCK
+
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
+
+      assert_config <<~YAML, string
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: ".gem_rbs_collection"
+        gemfile_lock_path: 'Gemfile.lock'
+        gems:
+          - name: ast
+            version: "2.4"
+            source:
+              name: ruby/gem_rbs_collection
+              remote: https://github.com/ruby/gem_rbs_collection.git
+              revision: cde6057e7546843ace6420c5783dd945c6ccda54
+              repo_dir: gems
+              type: git
+      YAML
+    end
+  end
+
+  def test_generate_lockfile_dependency_with_platform
+    mktmpdir do |tmpdir|
+      config_path = tmpdir / 'rbs_collection.yaml'
+      config_path.write [CONFIG, <<~YAML].join("\n")
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: '.gem_rbs_collection'
+        gems: []
+      YAML
+      gemfile_path = tmpdir / 'Gemfile'
+      gemfile_path.write <<~GEMFILE
+        source 'https://rubygems.org'
+
+        # Dependencies with platform may not appear in Gemfile.lock
+        gem "parser", platforms: [:jruby]
+      GEMFILE
+      gemfile_lock_path = tmpdir / 'Gemfile.lock'
+      gemfile_lock_path.write <<~GEMFILE_LOCK
+        GEM
+          remote: https://rubygems.org/
+          specs:
+
+        PLATFORMS
+          x86_64-linux
+
+        DEPENDENCIES
+          parser
+
+        BUNDLED WITH
+           2.2.0
+      GEMFILE_LOCK
+
+      definition = Bundler::Definition.build(gemfile_path, gemfile_lock_path, false)
+      _config, lockfile = RBS::Collection::Config.generate_lockfile(config_path: config_path, definition: definition)
+      string = YAML.dump(lockfile.to_lockfile)
+
+      assert_config <<~YAML, string
+        sources:
+          - type: git
+            name: ruby/gem_rbs_collection
+            remote: https://github.com/ruby/gem_rbs_collection.git
+            revision: cde6057e7546843ace6420c5783dd945c6ccda54
+            repo_dir: gems
+        path: ".gem_rbs_collection"
+        gemfile_lock_path: 'Gemfile.lock'
+      YAML
+    end
+  end
+
   private def assert_config(expected_str, actual_str)
     assert_equal YAML.load(expected_str), YAML.load(actual_str)
   end
