@@ -2228,8 +2228,8 @@ class Foo
   include Mod
 end
       DEF
-      RBS::Parser.parse_signature(rbs).each do |decl|
-        env << decl
+      RBS::Parser.parse_signature(rbs).tap do |buf, dirs, decls|
+        env.add_signature(buffer: buf, directives: dirs, decls: decls)
       end
       definition_builder = RBS::DefinitionBuilder.new(env: env.resolve_type_names)
       definition_builder.build_instance(TypeName("::Foo")).tap do |defn|
@@ -2289,4 +2289,216 @@ end
       end
     end
   end
+
+  def test_include_super_overload
+    SignatureManager.new do |manager|
+      manager.files.merge!(Pathname("foo.rbs") => <<~EOF)
+        interface _WithFoo
+          def foo: () -> void
+        end
+
+        module M1 : _WithFoo
+          def foo: (String) -> void
+                 | ...
+        end
+
+        class Foo1
+          def foo: () -> void
+                 | (Integer) -> void
+        end
+
+        class Foo < Foo1
+          include M1
+        end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::M1")).tap do |definition|
+          assert_equal(
+            [
+              parse_method_type("(::String) -> void"),
+              parse_method_type("() -> void"),
+            ],
+            definition.methods[:foo].defs.map(&:type)
+          )
+        end
+
+        builder.build_instance(type_name("::Foo")).tap do |definition|
+          assert_equal(
+            [
+              parse_method_type("(::String) -> void"),
+              parse_method_type("() -> void"),
+              parse_method_type("(::Integer) -> void")
+            ],
+            definition.methods[:foo].defs.map(&:type)
+          )
+        end
+      end
+    end
+  end
+
+  def test_build_instance_alias
+    SignatureManager.new do |manager|
+      manager.files[Pathname("foo.rbs")] = <<~EOF
+        class A
+        end
+
+        class B = A
+
+        module K = Kernel
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::B")).tap do |definition|
+          assert_instance_of Definition, definition
+          assert_equal type_name("::A"), definition.type_name
+        end
+
+        builder.build_instance(type_name("::K")).tap do |definition|
+          assert_instance_of Definition, definition
+          assert_equal type_name("::Kernel"), definition.type_name
+        end
+      end
+    end
+  end
+
+  def test_build_singleton_alias
+    SignatureManager.new do |manager|
+      manager.files[Pathname("foo.rbs")] = <<~EOF
+        class A
+          def initialize: (String) -> void
+        end
+
+        class B = A
+
+        module K = Kernel
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_singleton(type_name("::B")).tap do |definition|
+          assert_instance_of Definition, definition
+          assert_equal type_name("::A"), definition.type_name
+
+          assert_equal [parse_method_type("(::String) -> ::A")], definition.methods[:new].defs.map(&:type)
+        end
+
+        builder.build_singleton(type_name("::K")).tap do |definition|
+          assert_instance_of Definition, definition
+          assert_equal type_name("::Kernel"), definition.type_name
+        end
+      end
+    end
+  end
+
+  def test_build_instance_interface_mixin_transitive
+    SignatureManager.new do |manager|
+      manager.files[Pathname("foo.rbs")] = <<~EOF
+        interface _X
+          def x: () -> void
+
+          include _Y
+        end
+
+        interface _Y
+          def y: () -> void
+
+          include _Z
+        end
+
+        interface _Z
+          def z: () -> void
+        end
+
+        class A
+          include _X
+          extend _X
+        end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::A")).tap do |definition|
+          assert_instance_of Definition, definition
+
+          assert_operator Set[:x, :y, :z], :<=, Set.new(definition.methods.keys)
+        end
+      end
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_singleton(type_name("::A")).tap do |definition|
+          assert_instance_of Definition, definition
+
+          assert_operator Set[:x, :y, :z], :<=, Set.new(definition.methods.keys)
+        end
+      end
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_interface(type_name("::_X")).tap do |definition|
+          assert_instance_of Definition, definition
+
+          assert_operator Set[:x, :y, :z], :<=, Set.new(definition.methods.keys)
+        end
+      end
+    end
+  end
+
+  def test_mixin_method_owners
+    SignatureManager.new do |manager|
+      manager.files[Pathname("foo.rbs")] = <<~EOF
+        class A
+          include B
+        end
+
+        module B
+          def b: () -> void
+
+          attr_reader c: String
+
+          alias d b
+
+          def __id__: () -> void
+                    | ...
+        end
+      EOF
+
+      manager.build do |env|
+        builder = DefinitionBuilder.new(env: env)
+
+        builder.build_instance(type_name("::A")).tap do |definition|
+          assert_instance_of Definition, definition
+
+          definition.methods[:b].tap do |method|
+            assert_equal [TypeName("::B")], method.defs.map(&:defined_in)
+            assert_equal [TypeName("::B")], method.defs.map(&:implemented_in)
+          end
+
+          definition.methods[:c].tap do |method|
+            assert_equal [TypeName("::B")], method.defs.map(&:defined_in)
+            assert_equal [TypeName("::B")], method.defs.map(&:implemented_in)
+          end
+
+          definition.methods[:d].tap do |method|
+            assert_equal [TypeName("::B")], method.defs.map(&:defined_in)
+            assert_equal [TypeName("::B")], method.defs.map(&:implemented_in)
+          end
+
+          definition.methods[:__id__].tap do |method|
+            assert_equal [TypeName("::B"), TypeName("::Object")], method.defs.map(&:defined_in)
+            assert_equal [TypeName("::B"), TypeName("::B")], method.defs.map(&:implemented_in)
+          end
+        end
+      end
+    end
+  end
 end
