@@ -224,7 +224,7 @@ end
               type: git
       YAML
       RBS::Collection::Installer.new(lockfile_path: lockfile_path, stdout: StringIO.new).install_from_lockfile
-      lock = RBS::Collection::Config.from_path(lockfile_path)
+      lock = RBS::Collection::Config::Lockfile.from_lockfile(lockfile_path: lockfile_path, data: YAML.load_file(lockfile_path))
 
       repo = RBS::Repository.new()
 
@@ -236,11 +236,11 @@ end
 
       assert_operator env.class_decls, :key?, TypeName("::AST")
       assert_operator env.class_decls, :key?, TypeName("::Rainbow")
-      assert repo.dirs.include? lock.repo_path
+      assert repo.dirs.include? lock.fullpath
     end
   end
 
-  def test_loading_from_rbs_collection_without_install
+  def test_loading_from_rbs_collection_git_source_without_install
     mktmpdir do |path|
       lockfile_path = path.join('rbs_collection.lock.yaml')
       lockfile_path.write(<<~YAML)
@@ -268,7 +268,36 @@ end
               repo_dir: gems
               type: git
       YAML
-      lock = RBS::Collection::Config.from_path(lockfile_path)
+      lock = RBS::Collection::Config::Lockfile.from_lockfile(lockfile_path: lockfile_path, data: YAML.load_file(lockfile_path.to_s))
+
+      repo = RBS::Repository.new()
+
+      loader = EnvironmentLoader.new(repository: repo)
+
+      assert_raises RBS::Collection::Config::CollectionNotAvailable do
+        loader.add_collection(lock)
+      end
+    end
+  end
+
+  def test_loading_from_rbs_collection_local_source_without_install
+    mktmpdir do |path|
+      lockfile_path = path.join('rbs_collection.lock.yaml')
+      lockfile_path.write(<<~YAML)
+        sources:
+          - type: local
+            name: the local source
+            path: path/to/local/source
+        path: '.gem_rbs_collection'
+        gems:
+          - name: ast
+            version: "2.4"
+            source:
+              type: local
+              name: the local source
+              path: path/to/local/source
+      YAML
+      lock = RBS::Collection::Config::Lockfile.from_lockfile(lockfile_path: lockfile_path, data: YAML.load_file(lockfile_path.to_s))
 
       repo = RBS::Repository.new()
 
