@@ -62,7 +62,7 @@ end
 
     assert_write parser.decls, <<-EOF
 class Hello
-  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped, ?f: ::Integer, **untyped g) ?{ () -> untyped } -> nil
+  def hello: (untyped a, ?::Integer b, *untyped c, untyped d, e: untyped, ?f: ::Integer, **untyped g) { () -> untyped } -> nil
 
   def self.world: () { (untyped, untyped, untyped, x: untyped, y: untyped) -> untyped } -> untyped
 
@@ -233,6 +233,38 @@ end
     EOF
   end
 
+  def test_defs_return_type_with_block_optional
+      parser = RB.new
+
+      rb = <<~'EOR'
+        class Hello
+          def with_optional_block1
+            # `block_given?` call makes the block optional
+            if block_given?
+              yield 1
+            end
+          end
+
+          def with_optional_block2(&block)
+            # testing block var makes the block optional
+            if block
+              yield 1
+            end
+          end
+        end
+      EOR
+
+      parser.parse(rb)
+
+      assert_write parser.decls, <<~EOF
+        class Hello
+          def with_optional_block1: () ?{ (untyped) -> untyped } -> (untyped | nil)
+
+          def with_optional_block2: () ?{ (untyped) -> untyped } -> (untyped | nil)
+        end
+      EOF
+    end
+
   def test_defs_return_type_with_if
     parser = RB.new
 
@@ -907,11 +939,19 @@ end
 
       parser.parse(rb)
 
-      assert_write parser.decls, <<~RBS
+      if support_argument_forwarding_with_anonymous_kwrest?
+        assert_write parser.decls, <<~RBS
+module M
+  def foo: (*untyped, **untyped **) ?{ () -> untyped } -> nil
+end
+        RBS
+      else
+        assert_write parser.decls, <<~RBS
 module M
   def foo: (*untyped) ?{ () -> untyped } -> nil
 end
-      RBS
+        RBS
+      end
     end
   end
 
@@ -932,4 +972,13 @@ end
       RBS
     end
   end
+
+  private
+
+  def support_argument_forwarding_with_anonymous_kwrest?
+    eval("def foo(...); bar(**); end")
+    true
+  rescue Exception
+    false
+  end
 end
