@@ -15,12 +15,12 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
   include TestHelper
 
   def test_type_alias
-    Parser.parse_signature("type Steep::foo = untyped").yield_self do |decls|
+    Parser.parse_signature("type Steep::foo = untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       type_decl = decls[0]
 
-      assert_instance_of Declarations::Alias, type_decl
+      assert_instance_of Declarations::TypeAlias, type_decl
       assert_equal TypeName.new(name: :foo, namespace: Namespace.parse("Steep")), type_decl.name
       assert_equal [], type_decl.type_params.each.map(&:name)
       assert_equal Types::Bases::Any.new(location: nil), type_decl.type
@@ -35,27 +35,27 @@ class RBS::SignatureParsingTest < Test::Unit::TestCase
   end
 
   def test_type_alias_generic
-    Parser.parse_signature(<<RBS).yield_self do |decls|
+    Parser.parse_signature(<<RBS).yield_self do |_, _, decls|
 type optional[A] = A?
 RBS
       assert_equal 1, decls.size
 
       type_decl = decls[0]
 
-      assert_instance_of Declarations::Alias, type_decl
+      assert_instance_of Declarations::TypeAlias, type_decl
       assert_equal TypeName("optional"), type_decl.name
       assert_equal [:A], type_decl.type_params.each.map(&:name)
       assert_equal parse_type("A?", variables: [:A]), type_decl.type
       assert_equal "[A]", type_decl.location[:type_params].source
     end
 
-    Parser.parse_signature(<<RBS).yield_self do |decls|
+    Parser.parse_signature(<<RBS).tap do |_, _, decls|
 class Foo[A]
   type bar = B
 end
 RBS
       decls[0].members[0].tap do |type_decl|
-        assert_instance_of Declarations::Alias, type_decl
+        assert_instance_of Declarations::TypeAlias, type_decl
         assert_equal TypeName("bar"), type_decl.name
         assert_equal [], type_decl.type_params.each.map(&:name)
         assert_instance_of Types::ClassInstance, type_decl.type
@@ -65,7 +65,7 @@ RBS
   end
 
   def test_type_alias_generic_variance
-    Parser.parse_signature(<<RBS).yield_self do |decls|
+    Parser.parse_signature(<<RBS).yield_self do |_, _, decls|
 type x[T] = ^(T) -> void
 
 type y[unchecked out T] = ^(T) -> void
@@ -73,7 +73,7 @@ RBS
       assert_equal 2, decls.size
 
       decls[0].tap do |type_decl|
-        assert_instance_of Declarations::Alias, type_decl
+        assert_instance_of Declarations::TypeAlias, type_decl
 
         type_decl.type_params[0].tap do |param|
           assert_equal :T, param.name
@@ -83,7 +83,7 @@ RBS
       end
 
       decls[1].tap do |type_decl|
-        assert_instance_of Declarations::Alias, type_decl
+        assert_instance_of Declarations::TypeAlias, type_decl
 
         type_decl.type_params[0].tap do |param|
           assert_equal :T, param.name
@@ -95,7 +95,7 @@ RBS
   end
 
   def test_constant
-    Parser.parse_signature("FOO: untyped").yield_self do |decls|
+    Parser.parse_signature("FOO: untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       const_decl = decls[0]
@@ -106,7 +106,7 @@ RBS
       assert_equal "FOO: untyped", const_decl.location.source
     end
 
-    Parser.parse_signature("::BAR: untyped").yield_self do |decls|
+    Parser.parse_signature("::BAR: untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       const_decl = decls[0]
@@ -117,7 +117,7 @@ RBS
       assert_equal "::BAR: untyped", const_decl.location.source
     end
 
-    Parser.parse_signature("FOO : untyped").yield_self do |decls|
+    Parser.parse_signature("FOO : untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       const_decl = decls[0]
@@ -128,7 +128,7 @@ RBS
       assert_equal "FOO : untyped", const_decl.location.source
     end
 
-    Parser.parse_signature("::BAR : untyped").yield_self do |decls|
+    Parser.parse_signature("::BAR : untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       const_decl = decls[0]
@@ -141,7 +141,7 @@ RBS
   end
 
   def test_global
-    Parser.parse_signature("$FOO: untyped").yield_self do |decls|
+    Parser.parse_signature("$FOO: untyped").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       global_decl = decls[0]
@@ -154,7 +154,7 @@ RBS
   end
 
   def test_interface
-    Parser.parse_signature("interface _Each[A, B] end").yield_self do |decls|
+    Parser.parse_signature("interface _Each[A, B] end").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       interface_decl = decls[0]
@@ -166,7 +166,7 @@ RBS
       assert_equal "interface _Each[A, B] end", interface_decl.location.source
     end
 
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       interface _Each[A, B]
         #
         # Yield all elements included in `self`.
@@ -191,26 +191,27 @@ RBS
       interface_decl.members[0].yield_self do |def_member|
         assert_instance_of Members::MethodDefinition, def_member
         assert_equal :count, def_member.name
-        assert_equal 3, def_member.types.size
+        assert_equal 3, def_member.overloads.size
 
-        def_member.types.yield_self do |t1, t2, t3|
-          assert_empty t1.type_params
-          assert_nil t1.block
-          assert_equal "-> Integer", t1.location.source
+        def_member.overloads.yield_self do |o1, o2, o3|
+          assert_empty o1.annotations
+          assert_empty o1.method_type.type_params
+          assert_nil o1.method_type.block
+          assert_equal "-> Integer", o1.method_type.location.source
 
-          assert_empty t2.type_params
-          assert_equal 1, t2.type.required_positionals.size
-          assert_nil t2.block
-          assert_equal "(untyped) -> Integer", t2.location.source
+          assert_empty o2.method_type.type_params
+          assert_equal 1, o2.method_type.type.required_positionals.size
+          assert_nil o2.method_type.block
+          assert_equal "(untyped) -> Integer", o2.method_type.location.source
 
           assert_equal(
             [AST::TypeParam.new(name: :X, variance: :invariant, upper_bound: nil, location: nil)],
-            t3.type_params
+            o3.method_type.type_params
           )
-          assert_instance_of Types::Block, t3.block
-          assert_instance_of Types::Variable, t3.block.type.required_positionals[0].type
-          assert_instance_of Types::Variable, t3.block.type.return_type
-          assert_equal "[X] { (A) -> X } -> Integer", t3.location.source
+          assert_instance_of Types::Block, o3.method_type.block
+          assert_instance_of Types::Variable, o3.method_type.block.type.required_positionals[0].type
+          assert_instance_of Types::Variable, o3.method_type.block.type.return_type
+          assert_equal "[X] { (A) -> X } -> Integer", o3.method_type.location.source
         end
       end
 
@@ -239,7 +240,7 @@ RBS
   end
 
   def test_module
-    Parser.parse_signature("module Enumerable[A, B] end").yield_self do |decls|
+    Parser.parse_signature("module Enumerable[A, B] end").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       module_decl = decls[0]
@@ -252,7 +253,7 @@ RBS
       assert_equal "module Enumerable[A, B] end", module_decl.location.source
     end
 
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       module Enumerable[A, B] : _Each
         @foo: String
         self.@bar: Integer
@@ -429,7 +430,7 @@ RBS
   end
 
   def test_module_selfs
-    Parser.parse_signature(<<-RBS).yield_self do |decls|
+    Parser.parse_signature(<<-RBS).tap do |_, _, decls|
 module Enumerable[A, B] : _Each, Object
 end
     RBS
@@ -450,7 +451,7 @@ end
   end
 
   def test_class
-    Parser.parse_signature("class Array[A] end").yield_self do |decls|
+    Parser.parse_signature("class Array[A] end").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       decls[0].yield_self do |class_decl|
@@ -461,7 +462,7 @@ end
       end
     end
 
-    Parser.parse_signature("class ::Array[A] < Object[A] end").yield_self do |decls|
+    Parser.parse_signature("class ::Array[A] < Object[A] end").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       decls[0].yield_self do |class_decl|
@@ -477,7 +478,7 @@ end
   end
 
   def test_method_definition
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo[X, Y]
         def foo: -> Integer
                | ?{ -> void } -> Integer
@@ -491,22 +492,22 @@ end
 
         assert_instance_of Members::MethodDefinition, decl.members[0]
         decl.members[0].yield_self do |m|
-          assert_equal 3, m.types.size
+          assert_equal 3, m.overloads.size
 
-          m.types[0].yield_self do |ty|
+          m.overloads[0].method_type.yield_self do |ty|
             assert_instance_of MethodType, ty
             assert_equal "-> Integer", ty.location.source
             assert_nil ty.block
           end
 
-          m.types[1].yield_self do |ty|
+          m.overloads[1].method_type.yield_self do |ty|
             assert_instance_of MethodType, ty
             assert_equal "?{ -> void } -> Integer", ty.location.source
             assert_instance_of Types::Block, ty.block
             refute ty.block.required
           end
 
-          m.types[2].yield_self do |ty|
+          m.overloads[2].method_type.yield_self do |ty|
             assert_instance_of MethodType, ty
             assert_equal "[A] () { (String, ?Object, *Float, Symbol, foo: bool, ?bar: untyped, **Y) -> X } -> A", ty.location.source
             assert_instance_of Types::Block, ty.block
@@ -529,7 +530,7 @@ end
   end
 
   def test_private_public
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo
         public
         private
@@ -553,7 +554,7 @@ end
   end
 
   def test_private_public_def
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo
         public def foo: () -> void
         private def bar: () -> void
@@ -598,7 +599,7 @@ end
   end
 
   def test_private_public_attr
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo
         public attr_reader foo: String
         private attr_reader bar: String
@@ -655,7 +656,7 @@ end
   end
 
   def test_alias
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo
         def foo: -> String
         alias bar foo
@@ -686,7 +687,7 @@ end
   end
 
   def test_method_names
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       interface _Foo
         def class: -> String
         def void: -> String
@@ -778,7 +779,7 @@ end
   end
 
   def test_annotation_on_declaration
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       %a(foo)
       %a[hello world]
       class Hello end
@@ -818,7 +819,7 @@ end
       end
 
       decls[3].yield_self do |decl|
-        assert_instance_of Declarations::Alias, decl
+        assert_instance_of Declarations::TypeAlias, decl
 
         assert_equal "bar is okay", decl.annotations[0].string
         assert_equal "%a|bar is okay|", decl.annotations[0].location.source
@@ -827,7 +828,7 @@ end
   end
 
   def test_attributes
-    Parser.parse_signature(<<~SIG).tap do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Hello
         attr_reader a: Integer
         attr_writer b(@B): String
@@ -898,7 +899,7 @@ end
   end
 
   def test_annotations_on_members
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Hello
         %a{noreturn}
         def foo: () -> untyped
@@ -944,8 +945,28 @@ end
     end
   end
 
+  def test_annotations_on_overload
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
+      class Hello
+        def foo: %a{noreturn} () -> void
+               | %a{implicitly-returns-nil} %a{primitive:is_a?} (Class) -> bool
+      end
+    SIG
+
+      decls[0].members[0].yield_self do |m|
+        assert_instance_of Members::MethodDefinition, m
+
+        assert_equal ["noreturn"], m.overloads[0].annotations.map(&:string)
+        assert_equal ["implicitly-returns-nil", "primitive:is_a?"], m.overloads[1].annotations.map(&:string)
+
+        assert_equal "() -> void", m.overloads[0].method_type.location.source
+        assert_equal "(Class) -> bool", m.overloads[1].method_type.location.source
+      end
+    end
+  end
+
   def test_prepend
-    Parser.parse_signature(<<~SIG).yield_self do |decls|
+    Parser.parse_signature(<<~SIG).tap do |_, _, decls|
       class Foo
         prepend Foo
       end
@@ -992,7 +1013,7 @@ end
   end
 
   def test_class_comment
-    Parser.parse_signature(<<-EOF).yield_self do |foo_decl,bar_decl|
+    Parser.parse_signature(<<-EOF).yield_self do |_, _, (foo_decl,bar_decl)|
 # This is a class.
 # Foo Bar Baz.
 class Foo
@@ -1020,7 +1041,7 @@ EOF
   end
 
   def test_member_comment
-    Parser.parse_signature(<<-EOF).yield_self do |foo_decl,|
+    Parser.parse_signature(<<-EOF).yield_self do |_, _, (foo_decl)|
 # This is a class.
 # Foo Bar Baz.
 class Foo
@@ -1066,7 +1087,7 @@ EOF
   end
 
   def test_code_comment
-    Parser.parse_signature(<<-EOF).yield_self do |foo_decl,|
+    Parser.parse_signature(<<-EOF).yield_self do |_, _, (foo_decl)|
 # Passes each element of the collection to the given block. The method
 # returns `true` if the block never returns `false` or `nil` . If the
 # block is not given, Ruby adds an implicit block of `{ |obj| obj }` which
@@ -1108,7 +1129,7 @@ EOF
   end
 
   def test_comment_without_leading_space
-    Parser.parse_signature(<<-EOF).yield_self do |foo_decl,|
+    Parser.parse_signature(<<-EOF).yield_self do |_, _, (foo_decl)|
 #This is a class.
 class Foo
   #This is a method.
@@ -1157,7 +1178,7 @@ end
   end
 
   def test_module_type_param_variance
-    Parser.parse_signature("interface _Each[A, out B, unchecked in C] end").yield_self do |decls|
+    Parser.parse_signature("interface _Each[A, out B, unchecked in C] end").tap do |_, _, decls|
       assert_equal 1, decls.size
 
       interface_decl = decls[0]
@@ -1183,7 +1204,7 @@ end
   end
 
   def test_mame
-    Parser.parse_signature(<<EOF).yield_self do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 # h –
 class Exception < Object
 end
@@ -1194,7 +1215,7 @@ EOF
   end
 
   def test_decl_in_module
-    Parser.parse_signature(<<EOF).yield_self do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 module Steep
   VERSION: String
   type t = AST::Types::Base | AST::Types::Proc
@@ -1214,7 +1235,7 @@ EOF
       assert_equal 6, mod.members.size
 
       assert_instance_of Declarations::Constant, mod.members[0]
-      assert_instance_of Declarations::Alias, mod.members[1]
+      assert_instance_of Declarations::TypeAlias, mod.members[1]
       assert_instance_of Members::MethodDefinition, mod.members[2]
       assert_instance_of Declarations::Class, mod.members[3]
       assert_instance_of Declarations::Module, mod.members[4]
@@ -1226,7 +1247,7 @@ EOF
   end
 
   def test_overload_def
-    Parser.parse_signature(<<EOF).yield_self do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 module Steep
   def to_s: (Integer) -> String | ...
   def to_i: () -> Integer
@@ -1234,18 +1255,18 @@ end
 EOF
       decls[0].members[0].tap do |member|
         assert_instance_of Members::MethodDefinition, member
-        assert_predicate member, :overload?
+        assert_predicate member, :overloading?
       end
 
       decls[0].members[1].tap do |member|
         assert_instance_of Members::MethodDefinition, member
-        refute_predicate member, :overload?
+        refute_predicate member, :overloading?
       end
     end
   end
 
   def test_generics_type_parameter
-    Parser.parse_signature(<<EOF).yield_self do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 module A[T]
   module B
     def foo: () -> void
@@ -1256,13 +1277,13 @@ end
 EOF
       decls[0].members[1].tap do |member|
         assert_instance_of Members::MethodDefinition, member
-        assert_instance_of Types::Variable, member.types[0].type.return_type
+        assert_instance_of Types::Variable, member.overloads[0].method_type.type.return_type
       end
     end
   end
 
   def test_proc
-    Parser.parse_signature(<<EOF).tap do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 module A
   def bar: () -> ^->Integer
 end
@@ -1270,7 +1291,7 @@ EOF
 
       decls[0].members[0].tap do |member|
         assert_instance_of Members::MethodDefinition, member
-        member.types[0].type.return_type.tap do |return_type|
+        member.overloads[0].method_type.type.return_type.tap do |return_type|
           assert_instance_of Types::Proc, return_type
           assert_instance_of Types::ClassInstance, return_type.type.return_type
         end
@@ -1290,13 +1311,13 @@ EOF
   end
 
   def test_empty
-    Parser.parse_signature("").tap do |decls|
+    Parser.parse_signature("").tap do |_, _, decls|
       assert_empty decls
     end
   end
 
   def test_module_self_syntax
-    Parser.parse_signature(<<EOF).tap do |decls|
+    Parser.parse_signature(<<EOF).tap do |_, _, decls|
 module Foo: Object
 end
 
@@ -1321,7 +1342,7 @@ EOF
   end
 
   def test_method_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 module A
   def foo: () -> void
 
@@ -1335,7 +1356,7 @@ end
         assert_equal "def", foo.location[:keyword].source
         assert_equal "foo", foo.location[:name].source
         assert_nil foo.location[:kind]
-        assert_nil foo.location[:overload]
+        assert_nil foo.location[:overloading]
       end
 
       decls[0].members[1].tap do |foo|
@@ -1344,13 +1365,13 @@ end
         assert_equal "def", foo.location[:keyword].source
         assert_equal "bar", foo.location[:name].source
         assert_equal "self?.", foo.location[:kind].source
-        assert_equal "...", foo.location[:overload].source
+        assert_equal "...", foo.location[:overloading].source
       end
     end
   end
 
   def test_var_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 module A
   @foo: Integer
   self.@bar: String
@@ -1385,7 +1406,7 @@ end
   end
 
   def test_attribute_location
-    Parser.parse_signature(<<-RBS).tap do |decls|
+    Parser.parse_signature(<<-RBS).tap do |_, _, decls|
 module A
   attr_reader reader1: String
   attr_reader reader2 : String
@@ -1463,7 +1484,7 @@ end
       end
     end
 
-    Parser.parse_signature(<<-RBS).tap do |decls|
+    Parser.parse_signature(<<-RBS).tap do |_, _, decls|
 module A
   attr_writer attr1: String
   attr_writer attr2 : String
@@ -1541,7 +1562,7 @@ end
       end
     end
 
-    Parser.parse_signature(<<-RBS).tap do |decls|
+    Parser.parse_signature(<<-RBS).tap do |_, _, decls|
 module A
   attr_accessor attr1: String
   attr_accessor attr2 : String
@@ -1621,7 +1642,7 @@ end
   end
 
   def test_alias_location
-    Parser.parse_signature(<<-RBS).tap do |decls|
+    Parser.parse_signature(<<-RBS).tap do |_, _, decls|
 module A
   alias foo bar
   alias self.foo self.bar
@@ -1651,7 +1672,7 @@ end
   end
 
   def test_mixin_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 module A
   include _Foo
   include _Bar[String]
@@ -1714,7 +1735,7 @@ end
   end
 
   def test_interface_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 interface _A
 end
 
@@ -1758,7 +1779,7 @@ end
   end
 
   def test_module_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 module A
 end
 
@@ -1803,11 +1824,14 @@ module C: BasicObject end
   end
 
   def test_class_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 class A
 end
 
 class B[X] < Foo[X]
+end
+
+class C < Bar
 end
     EOF
       decls[0].tap do |decl|
@@ -1831,12 +1855,29 @@ end
 
         assert_equal "Foo", decl.super_class.location[:name].source
         assert_equal "[X]", decl.super_class.location[:args].source
+        assert_equal [4, 13], decl.super_class.location.start_loc
+        assert_equal [4, 19], decl.super_class.location.end_loc
+      end
+
+      decls[2].tap do |decl|
+        assert_instance_of Location, decl.location
+
+        assert_equal "class", decl.location[:keyword].source
+        assert_equal "C", decl.location[:name].source
+        assert_equal "end", decl.location[:end].source
+        assert_nil decl.location[:type_params]
+        assert_equal "<", decl.location[:lt].source
+
+        assert_equal "Bar", decl.super_class.location[:name].source
+        assert_nil decl.super_class.location[:args]
+        assert_equal [7, 10], decl.super_class.location.start_loc
+        assert_equal [7, 13], decl.super_class.location.end_loc
       end
     end
   end
 
   def test_constant_global_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 X: String
 A::B : String
 $B: Integer
@@ -1865,7 +1906,7 @@ $B: Integer
   end
 
   def test_type_alias_location
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 type foo = Integer
     EOF
       decls[0].tap do |decl|
@@ -1919,7 +1960,7 @@ type x = Foo::_bar
   end
 
   def test_singleton_member_type_variables
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 class Foo[A]
   @foo: A
 
@@ -1953,9 +1994,9 @@ end
         assert_instance_of Types::ClassInstance, decl.members[1].type
         assert_instance_of Types::ClassInstance, decl.members[2].type
 
-        assert_instance_of Types::Variable, decl.members[3].types[0].type.return_type
-        assert_instance_of Types::ClassInstance, decl.members[4].types[0].type.return_type
-        assert_instance_of Types::ClassInstance, decl.members[5].types[0].type.return_type
+        assert_instance_of Types::Variable, decl.members[3].overloads[0].method_type.type.return_type
+        assert_instance_of Types::ClassInstance, decl.members[4].overloads[0].method_type.type.return_type
+        assert_instance_of Types::ClassInstance, decl.members[5].overloads[0].method_type.type.return_type
 
         assert_instance_of Types::ClassInstance, decl.members[6].type
         assert_instance_of Types::ClassInstance, decl.members[7].type
@@ -1969,7 +2010,7 @@ end
   end
 
   def test_generics_bound
-    Parser.parse_signature(<<-EOF).tap do |decls|
+    Parser.parse_signature(<<-EOF).tap do |_, _, decls|
 class Foo[X < _Each[Y], Y]
   def foo: [X < Array[Y]] (X) -> X
 end
@@ -1993,7 +2034,7 @@ end
         end
 
         decl.members[0].tap do |member|
-          member.types[0].type_params[0].tap do |param|
+          member.overloads[0].method_type.type_params[0].tap do |param|
             assert_equal :X, param.name
             assert_equal :invariant, param.variance
             refute_predicate param, :unchecked?
@@ -2012,4 +2053,82 @@ end
             EOF
     end
   end
+
+  def test_module_alias_decl
+    Parser.parse_signature(<<~EOF).tap do |_, _, decls|
+        module RBS::Kernel = Kernel
+      EOF
+
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::ModuleAlias, decl
+
+        assert_equal TypeName("RBS::Kernel"), decl.new_name
+        assert_equal TypeName("Kernel"), decl.old_name
+        assert_equal "module", decl.location[:keyword].source
+        assert_equal "RBS::Kernel", decl.location[:new_name].source
+        assert_equal "=", decl.location[:eq].source
+        assert_equal "Kernel", decl.location[:old_name].source
+      end
+    end
+  end
+
+  def test_class_alias_decl
+    Parser.parse_signature(<<~EOF).tap do |_, _, decls|
+        class RBS::Object = Object
+      EOF
+
+      decls[0].tap do |decl|
+        assert_instance_of Declarations::ClassAlias, decl
+
+        assert_equal TypeName("RBS::Object"), decl.new_name
+        assert_equal TypeName("Object"), decl.old_name
+        assert_equal "class", decl.location[:keyword].source
+        assert_equal "RBS::Object", decl.location[:new_name].source
+        assert_equal "=", decl.location[:eq].source
+        assert_equal "Object", decl.location[:old_name].source
+      end
+    end
+  end
+
+  def test_use_directive
+    Parser.parse_signature(<<~RBS).tap do |_, dirs, _|
+      use RBS::Namespace as NS
+      use RBS::TypeName, RBS::AST::Declarations::*
+
+      module Baz
+      end
+
+      class Foo
+      end
+    RBS
+
+      assert_equal 2, dirs.size
+
+      dirs[0].tap do |use|
+        assert_equal 1, use.clauses.size
+        assert_equal TypeName("RBS::Namespace"), use.clauses[0].type_name
+        assert_equal :NS, use.clauses[0].new_name
+      end
+
+      dirs[1].tap do |use|
+        assert_equal 2, use.clauses.size
+
+        assert_equal TypeName("RBS::TypeName"), use.clauses[0].type_name
+        assert_nil use.clauses[0].new_name
+
+        assert_equal Namespace("RBS::AST::Declarations::"), use.clauses[1].namespace
+      end
+    end
+  end
+
+  def test_use_directive_error
+    assert_raises do
+      Parser.parse_signature(<<~RBS)
+        module Baz
+        end
+
+        use RBS::Namespace as NS
+      RBS
+    end
+  end
 end
