@@ -19,25 +19,37 @@ class FileUtilsSingletonTest < Test::Unit::TestCase
   testing "singleton(::FileUtils)"
 
   def test_cd
-    assert_send_type  "(String) -> void",
-                      FileUtils, :cd, __dir__
-    assert_send_type  "(ToStr) -> void",
-                      FileUtils, :cd, ToStr.new(__dir__)
-    assert_send_type  "(ToPath, verbose: bool) -> void",
-                      FileUtils, :cd, ToPath.new(__dir__), verbose: false
-    assert_send_type  "(String) { (String) -> Integer } -> Integer",
-                      FileUtils, :cd, __dir__ do |dir| 1 end
-    assert_send_type  "(ToStr) { (String) -> Integer } -> Integer",
-                      FileUtils, :cd, ToStr.new(__dir__) do |dir| 1 end
-    assert_send_type  "(ToPath, verbose: nil) { (String) -> Integer } -> Integer",
-                      FileUtils, :cd, ToPath.new(__dir__), verbose: nil do |dir| 1 end
+    dir = Dir.pwd
+
+    begin
+      assert_send_type  "(String) -> void",
+                        FileUtils, :cd, __dir__
+      assert_send_type  "(ToStr) -> void",
+                        FileUtils, :cd, ToStr.new(__dir__)
+      assert_send_type  "(ToPath, verbose: bool) -> void",
+                        FileUtils, :cd, ToPath.new(__dir__), verbose: false
+      assert_send_type  "(String) { (String) -> Integer } -> Integer",
+                        FileUtils, :cd, __dir__ do |dir| 1 end
+      assert_send_type  "(ToStr) { (String) -> Integer } -> Integer",
+                        FileUtils, :cd, ToStr.new(__dir__) do |dir| 1 end
+      assert_send_type  "(ToPath, verbose: nil) { (String) -> Integer } -> Integer",
+                        FileUtils, :cd, ToPath.new(__dir__), verbose: nil do |dir| 1 end
+    ensure
+      Dir.chdir dir
+    end
   end
 
   def test_chdir
-    assert_send_type  "(String) -> void",
-                      FileUtils, :chdir, __dir__
-    assert_send_type  "(String) { (String) -> Integer } -> Integer",
-                      FileUtils, :chdir, __dir__ do |dir| 1 end
+    dir = Dir.pwd
+
+    begin
+      assert_send_type  "(String) -> void",
+                        FileUtils, :chdir, __dir__
+      assert_send_type  "(String) { (String) -> Integer } -> Integer",
+                        FileUtils, :chdir, __dir__ do |dir| 1 end
+    ensure
+      Dir.chdir(dir)
+    end
   end
 
   def test_chmod
@@ -309,8 +321,8 @@ class FileUtilsSingletonTest < Test::Unit::TestCase
                         FileUtils, :ln_s, ToStr.new("src"), ToStr.new("dest"), noop: true
       assert_send_type  "(Array[String | ToStr | ToPath], String, noop: bool) -> void",
                         FileUtils, :ln_s, ["src", ToStr.new("src"), ToPath.new("src")], "dest_dir", noop: true
-      assert_send_type  "(ToPath, ToPath, force: nil, noop: bool, verbose: bool) -> void",
-                        FileUtils, :ln_s, ToPath.new("src"), ToPath.new("dest"), force: nil, noop: true, verbose: false
+      assert_send_type  "(ToPath, ToPath, force: nil, noop: bool, verbose: bool, target_directory: bool, relative: bool) -> void",
+                        FileUtils, :ln_s, ToPath.new("src"), ToPath.new("dest"), force: nil, noop: true, verbose: false, relative: false, target_directory: false
     end
   end
 
@@ -337,6 +349,22 @@ class FileUtilsSingletonTest < Test::Unit::TestCase
     end
   end
 
+  def test_ln_sr
+    in_tmpdir do
+      File.write "src", ""
+      Dir.mkdir "dest_dir"
+
+      assert_send_type  "(String, String) -> void",
+                        FileUtils, :ln_sr, "src", "dest"
+      assert_send_type  "(ToStr, ToStr, noop: bool) -> void",
+                        FileUtils, :ln_sr, ToStr.new("src"), ToStr.new("dest"), noop: true
+      assert_send_type  "(ToPath, ToPath, noop: bool) -> void",
+                        FileUtils, :ln_sr, ToPath.new("src"), ToPath.new("dest"), noop: true
+      assert_send_type  "(Array[String | ToStr | ToPath], String, noop: bool, verbose: bool, target_directory: false) -> void",
+                        FileUtils, :ln_sr, ["src", ToStr.new("src"), ToStr.new("src")], "dest_dir", noop: true, verbose: false, target_directory: false
+    end
+  end
+
   def test_mkdir
     in_tmpdir do
       assert_send_type  "(String) -> void",
@@ -601,13 +629,25 @@ class FileUtilsInstanceTest < Test::Unit::TestCase
   end
 
   def test_cd
-    assert_send_type  "(String) -> void",
-                      Foo.new, :cd, __dir__
+    dir = Dir.pwd
+
+    begin
+      assert_send_type  "(String) -> void",
+                        Foo.new, :cd, __dir__
+    ensure
+      Dir.chdir(dir)
+    end
   end
 
   def test_chdir
-    assert_send_type  "(String) -> void",
-                      Foo.new, :chdir, __dir__
+    dir = Dir.pwd
+
+    begin
+      assert_send_type  "(String) -> void",
+                        Foo.new, :chdir, __dir__
+    ensure
+      Dir.chdir(dir)
+    end
   end
 
   def test_chmod
