@@ -10,11 +10,11 @@ class IOSingletonTest < Test::Unit::TestCase
 
   def test_binread
     assert_send_type "(String) -> String",
-                     IO, :binread, File.expand_path(__FILE__, "../..")
+                     IO, :binread, File.expand_path(__FILE__)
     assert_send_type "(String, Integer) -> String",
-                     IO, :binread, File.expand_path(__FILE__, "../.."), 3
+                     IO, :binread, File.expand_path(__FILE__), 3
     assert_send_type "(String, Integer, Integer) -> String",
-                     IO, :binread, File.expand_path(__FILE__, "../.."), 3, 0
+                     IO, :binread, File.expand_path(__FILE__), 3, 0
   end
 
   def test_binwrite
@@ -35,14 +35,20 @@ class IOSingletonTest < Test::Unit::TestCase
 
   def test_open
     Dir.mktmpdir do |dir|
-      fd = IO.sysopen(File.expand_path(__FILE__, "../.."))
+      fd = IO.sysopen(File.expand_path(__FILE__))
 
       assert_send_type "(Integer) -> IO",
                        IO, :open, fd
       assert_send_type "(ToInt, String) -> IO",
                        IO, :open, ToInt.new(fd), "r"
-      assert_send_type "(Integer) { (IO) -> String } -> String",
-                       IO, :open, fd do |io| io.read end
+      assert_send_type "(Integer) { (IO) -> Integer } -> Integer",
+                       IO, :open, fd do |io| io.read.size end
+
+      fd = IO.sysopen(File.expand_path(__FILE__))
+      assert_send_type(
+        "(ToInt, path: String) -> IO",
+        IO, :open, ToInt.new(fd), path: "<<TEST>>"
+      )
     end
   end
 
@@ -89,30 +95,55 @@ class IOSingletonTest < Test::Unit::TestCase
     end
   end
 
+  def test_new
+    IO.sysopen(File.expand_path(__FILE__)).tap do |fd|
+      assert_send_type(
+        "(Integer) -> IO",
+        IO, :new, fd
+      )
+    end
+
+    IO.sysopen(File.expand_path(__FILE__)).tap do |fd|
+      assert_send_type(
+        "(ToInt, ToStr, path: ToStr) -> IO",
+        IO, :new, ToInt.new(fd), ToStr.new("r"), path: ToStr.new("<<TEST>>")
+      )
+    end
+
+    IO.sysopen(File.expand_path(__FILE__)).tap do |fd|
+      assert_send_type(
+        "(Integer, path: nil) -> IO",
+        IO, :new, fd, path: nil
+      )
+    end
+  end
+
   def test_select
-    r, w = IO.pipe
-    assert_send_type "(Array[IO], nil, nil, Float) -> nil",
-      IO, :select, [r], nil, nil, 0.5
-    assert_send_type "(nil, Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, nil, [w]
-    assert_send_type "(nil, Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, nil, [w], [r]
-    assert_send_type "(nil, Array[IO], Array[IO], Float) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, nil, [w], [r], 0.5
-    w.write("x")
-    assert_send_type "(Array[IO], nil, nil, Float) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, [r], nil, nil, 0.5
-    assert_send_type "(Array[IO], nil, nil, nil) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, [r], nil, nil, nil
-    assert_send_type "(Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, [r], [w]
-    assert_send_type "(Array[IO], Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, [r], [w], [r]
-    assert_send_type "(Array[IO], Array[IO], Array[IO], Float) -> [Array[IO], Array[IO], Array[IO]]",
-      IO, :select, [r], [w], [r], 0.5
-  ensure
-    r.close
-    w.close
+    if_ruby "3.0.0"..."3.2.0" do
+      r, w = IO.pipe
+      assert_send_type "(Array[IO], nil, nil, Float) -> nil",
+        IO, :select, [r], nil, nil, 0.5
+      assert_send_type "(nil, Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, nil, [w]
+      assert_send_type "(nil, Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, nil, [w], [r]
+      assert_send_type "(nil, Array[IO], Array[IO], Float) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, nil, [w], [r], 0.5
+      w.write("x")
+      assert_send_type "(Array[IO], nil, nil, Float) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, [r], nil, nil, 0.5
+      assert_send_type "(Array[IO], nil, nil, nil) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, [r], nil, nil, nil
+      assert_send_type "(Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, [r], [w]
+      assert_send_type "(Array[IO], Array[IO], Array[IO]) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, [r], [w], [r]
+      assert_send_type "(Array[IO], Array[IO], Array[IO], Float) -> [Array[IO], Array[IO], Array[IO]]",
+        IO, :select, [r], [w], [r], 0.5
+    ensure
+      r.close
+      w.close
+    end
   end
 end
 
@@ -133,7 +164,7 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_advise
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type "(Symbol) -> nil",
                        io, :advise, :normal
       assert_send_type "(Symbol) -> nil",
@@ -154,7 +185,7 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_autoclose=
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type "(bool) -> bool",
                        io, :autoclose=, true
       assert_send_type "(bool) -> bool",
@@ -167,14 +198,30 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_autoclose?
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type "() -> bool",
                        io, :autoclose?
     end
   end
 
+  def test_path
+    IO.open(IO.sysopen(File.expand_path(__FILE__)), path: "foo") do |io|
+      assert_send_type(
+        "() -> String",
+        io, :path
+      )
+    end
+
+    IO.open(IO.sysopen(File.expand_path(__FILE__)), path: nil) do |io|
+      assert_send_type(
+        "() -> nil",
+        io, :path
+      )
+    end
+  end
+
   def test_read
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type "() -> String",
                        io, :read
       assert_send_type "(Integer) -> String",
@@ -193,7 +240,7 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_readpartial
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type "(Integer) -> String",
                        io, :readpartial, 10
       assert_send_type "(Integer, String) -> String",
@@ -201,6 +248,46 @@ class IOInstanceTest < Test::Unit::TestCase
     end
   end
 
+  def test_timeout
+    io, _ = IO.pipe()
+
+    assert_send_type(
+      "(Integer) -> IO",
+      io, :timeout=, 1
+    )
+    assert_send_type(
+      "() -> Integer",
+      io, :timeout
+    )
+
+    assert_send_type(
+      "(Float) -> IO",
+      io, :timeout=, 1.2
+    )
+    assert_send_type(
+      "() -> Float",
+      io, :timeout
+    )
+
+    assert_send_type(
+      "(Rational) -> IO",
+      io, :timeout=, 3r
+    )
+    assert_send_type(
+      "() -> Rational",
+      io, :timeout
+    )
+
+    assert_send_type(
+      "(nil) -> IO",
+      io, :timeout=, nil
+    )
+    assert_send_type(
+      "() -> nil",
+      io, :timeout
+    )
+  end
+
   def test_write
     Dir.mktmpdir do |dir|
       File.open(File.join(dir, "some_file"), "w") do |io|
@@ -215,7 +302,7 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_close_on_exec
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type '() -> bool',
                        io, :close_on_exec?
       assert_send_type '(::Integer) -> untyped',
@@ -230,7 +317,7 @@ class IOInstanceTest < Test::Unit::TestCase
   end
 
   def test_sync
-    IO.open(IO.sysopen(File.expand_path(__FILE__, "../.."))) do |io|
+    IO.open(IO.sysopen(File.expand_path(__FILE__))) do |io|
       assert_send_type '() -> bool',
                        io, :sync
       assert_send_type '(::Integer) -> ::Integer',
