@@ -9,6 +9,13 @@ class ModuleSingletonTest < Test::Unit::TestCase
     assert_send_type "() -> Array[Module]",
                      Module, :used_modules
   end
+
+  def test_used_refinements
+    assert_send_type(
+      "() -> Array[Refinement]",
+      Module, :used_refinements
+    )
+  end
 end
 
 class ModuleInstanceTest < Test::Unit::TestCase
@@ -25,6 +32,13 @@ class ModuleInstanceTest < Test::Unit::TestCase
                      Foo, :refine, String do nil end
   end
 
+  def test_refinements
+    assert_send_type(
+      "() -> Array[Refinement]",
+      Foo, :refinements
+    )
+  end
+
   def test_const_source_location
     assert_send_type "(Symbol) -> [String, Integer]",
                      Foo, :const_source_location, :BAR
@@ -144,4 +158,107 @@ class ModuleInstanceTest < Test::Unit::TestCase
       mod, :public, :foo, "bar"
     )
   end
+
+  def test_attr
+    if RUBY_VERSION >= '3.0'
+      mod = Module.new
+      assert_send_type(
+        "(*Symbol | String arg0) -> Array[Symbol]",
+        mod, :attr, :foo
+      )
+    end
+  end
+
+  def test_attr_reader
+    if RUBY_VERSION >= '3.0'
+      mod = Module.new
+
+      assert_send_type(
+        "(Symbol) -> Array[Symbol]",
+        mod, :attr_reader, :foo
+      )
+
+      assert_send_type(
+        "(Symbol, Symbol) -> Array[Symbol]",
+        mod, :attr_reader, :foo, :bar
+      )
+
+      assert_send_type(
+        "(String) -> Array[Symbol]",
+        mod, :attr_reader, "foo"
+      )
+
+      assert_send_type(
+        "(String, String) -> Array[Symbol]",
+        mod, :attr_reader, "foo", "bar"
+      )
+
+      assert_send_type(
+        "(Symbol, String) -> Array[Symbol]",
+        mod, :attr_reader, :foo, "bar"
+      )
+    end
+  end
+
+  def test_attr_writer
+    if RUBY_VERSION >= '3.0'
+      mod = Module.new
+
+      assert_send_type(
+        "(Symbol) -> Array[Symbol]",
+        mod, :attr_writer, :foo
+      )
+
+      assert_send_type(
+        "(Symbol, Symbol) -> Array[Symbol]",
+        mod, :attr_writer, :foo, :bar
+      )
+
+      assert_send_type(
+        "(String) -> Array[Symbol]",
+        mod, :attr_writer, "foo"
+      )
+
+      assert_send_type(
+        "(String, String) -> Array[Symbol]",
+        mod, :attr_writer, "foo", "bar"
+      )
+
+      assert_send_type(
+        "(Symbol, String) -> Array[Symbol]",
+        mod, :attr_writer, :foo, "bar"
+      )
+    end
+  end
+
+  def test_attr_accessor
+    if RUBY_VERSION >= '3.0'
+      mod = Module.new
+
+      assert_send_type(
+        "(Symbol) -> Array[Symbol]",
+        mod, :attr_accessor, :foo
+      )
+
+      assert_send_type(
+        "(Symbol, Symbol) -> Array[Symbol]",
+        mod, :attr_accessor, :foo, :bar
+      )
+
+      assert_send_type(
+        "(String) -> Array[Symbol]",
+        mod, :attr_accessor, "foo"
+      )
+
+      assert_send_type(
+        "(String, String) -> Array[Symbol]",
+        mod, :attr_accessor, "foo", "bar"
+      )
+
+      assert_send_type(
+        "(Symbol, String) -> Array[Symbol]",
+        mod, :attr_accessor, :foo, "bar"
+      )
+    end
+  end
 end
