@@ -170,15 +170,18 @@ module TypeAssertions
       @target = nil
     end
 
-    def env
-      @env ||= begin
-                 loader = RBS::EnvironmentLoader.new
-                 (@libs || []).each do |lib|
-                   loader.add library: lib
-                 end
+    @@env_cache = {}
 
-                 RBS::Environment.from_loader(loader).resolve_type_names
-               end
+    def env
+      @env = @@env_cache[@libs] ||=
+        begin
+          loader = RBS::EnvironmentLoader.new
+          (@libs || []).each do |lib|
+            loader.add library: lib
+          end
+
+          RBS::Environment.from_loader(loader).resolve_type_names
+        end
     end
 
     def builder
@@ -399,6 +402,16 @@ class ToInt
   end
 end
 
+class ToF
+  def initialize(value = 0.1)
+    @value = value
+  end
+
+  def to_f
+    @value
+  end
+end
+
 class ToStr
   def initialize(value = "")
     @value = value
@@ -439,6 +452,16 @@ class ToPath
   end
 end
 
+class Each
+  def initialize(*args)
+    @args = args
+  end
+
+  def each(&block)
+    @args.each(&block)
+  end
+end
+
 class ToJson
 end
 
