@@ -25,9 +25,9 @@ type ty = String | Integer
       EOF
 
       manager.build do |env|
-        root = [Namespace.root]
+        root = nil
 
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
         validator.validate_type(parse_type("::Foo"), context: root)
@@ -83,9 +83,9 @@ type u_2 = string & u & Numeric
       EOF
 
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
-        env.alias_decls.each do |name, decl|
+        env.type_alias_decls.each do |name, decl|
           assert_raises RBS::RecursiveTypeAliasError do
             validator.validate_type_alias(entry: decl)
           end
@@ -106,10 +106,10 @@ type proc = ^(proc) -> proc
 type record = { foo: record }
       EOF
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
-        env.alias_decls.each do |name, entry|
+        env.type_alias_decls.each do |name, entry|
           validator.validate_type_alias(entry: entry)
         end
       end
@@ -127,17 +127,17 @@ type baz[out T] = ^(T) -> void
       EOF
 
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
-        validator.validate_type_alias(entry: env.alias_decls[type_name("::foo")])
+        validator.validate_type_alias(entry: env.type_alias_decls[type_name("::foo")])
 
         assert_raises RBS::NonregularTypeAliasError do
-          validator.validate_type_alias(entry: env.alias_decls[type_name("::bar")])
+          validator.validate_type_alias(entry: env.type_alias_decls[type_name("::bar")])
         end
 
         assert_raises RBS::InvalidVarianceAnnotationError do
-          validator.validate_type_alias(entry: env.alias_decls[type_name("::baz")])
+          validator.validate_type_alias(entry: env.type_alias_decls[type_name("::baz")])
         end
       end
     end
@@ -152,13 +152,13 @@ type bar[T < _Foo[S], S < _Bar[T]] = nil
       EOF
 
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
-        validator.validate_type_alias(entry: env.alias_decls[type_name("::foo")])
+        validator.validate_type_alias(entry: env.type_alias_decls[type_name("::foo")])
 
         error = assert_raises(RBS::CyclicTypeParameterBound) do
-          validator.validate_type_alias(entry: env.alias_decls[type_name("::bar")])
+          validator.validate_type_alias(entry: env.type_alias_decls[type_name("::bar")])
         end
 
         assert_equal error.type_name, TypeName("::bar")
@@ -177,10 +177,10 @@ type foo[unchecked out A, unchecked in B] = Foo[A, B]
       RBS
 
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
-        validator.validate_type_alias(entry: env.alias_decls[type_name("::foo")])
+        validator.validate_type_alias(entry: env.type_alias_decls[type_name("::foo")])
       end
     end
   end
@@ -192,19 +192,54 @@ type foo = bar
       RBS
 
       manager.build do |env|
-        resolver = RBS::TypeNameResolver.from_env(env)
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
         validator = RBS::Validator.new(env: env, resolver: resolver)
 
         # No error is raised.
-        validator.validate_type_alias(entry: env.alias_decls[type_name("::foo")])
+        validator.validate_type_alias(entry: env.type_alias_decls[type_name("::foo")])
 
         # Passing a block and validating the given type raises an error.
         assert_raises RBS::NoTypeFoundError do
-          validator.validate_type_alias(entry: env.alias_decls[type_name("::foo")]) do |type|
+          validator.validate_type_alias(entry: env.type_alias_decls[type_name("::foo")]) do |type|
             validator.validate_type(type, context: [])
           end
         end
       end
     end
   end
+
+  def test_validate_class_alias
+    SignatureManager.new do |manager|
+      manager.add_file("bar.rbs", <<-EOF)
+class Foo = Kernel
+
+module Bar = NoSuchClass
+
+class Baz = Baz
+      EOF
+
+      manager.build do |env|
+        resolver = RBS::Resolver::TypeNameResolver.new(env)
+        validator = RBS::Validator.new(env: env, resolver: resolver)
+
+        env.class_alias_decls[TypeName("::Foo")].tap do |entry|
+          assert_raises RBS::InconsistentClassModuleAliasError do
+            validator.validate_class_alias(entry: entry)
+          end
+        end
+
+        env.class_alias_decls[TypeName("::Bar")].tap do |entry|
+          assert_raises RBS::NoTypeFoundError do
+            validator.validate_class_alias(entry: entry)
+          end
+        end
+
+        env.class_alias_decls[TypeName("::Baz")].tap do |entry|
+          assert_raises RBS::CyclicClassAliasDefinitionError do
+            validator.validate_class_alias(entry: entry)
+          end
+        end
+      end
+    end
+  end
 end
